@@ -36,12 +36,12 @@ main()
     /* Open file and create group. */
     if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                             H5P_DEFAULT)) < 0) ERR;
-    if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+    if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
     /* Write an array of bools. */
     dims[0] = DIM1_LEN;
     if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-    if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+    if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                spaceid, H5P_DEFAULT)) < 0) ERR;
     if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                  bool_out) < 0) ERR;
@@ -69,19 +69,19 @@ main()
         float float_data_out[LAT_LEN][LON_LEN];
         hsize_t dims[NDIMS], max_dims[NDIMS];
         hsize_t dims_in[NDIMS], max_dims_in[NDIMS];
-        hsize_t start[MAX_DIMS], count[MAX_DIMS];
+        hsize_t start[MAX_DIMS], count[MAX_DIMS], ones[MAX_DIMS];
         int lat, lon;
 
         /* Set up some phoney data, 1 record's worth. In C, first
          * dimension varies most slowly. */
         for (lat = 0; lat < LAT_LEN; lat++)
             for (lon = 0; lon < LON_LEN; lon++)
-                float_data_out[lat][lon] = -666.666;
+                float_data_out[lat][lon] = -666.666f;
 
         /* Create file and group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP2_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP2_NAME, 0)) < 0) ERR;
 
         /* Create a space corresponding to these three dimensions. */
         dims[0] = 0;
@@ -100,9 +100,9 @@ main()
         if (H5Pset_chunk(cparmsid, NDIMS, dims) < 0) ERR;
 
         /* Create two variables which use this space. */
-        if ((pres_dsid = H5Dcreate(grpid, PRES_NAME, H5T_NATIVE_FLOAT,
+        if ((pres_dsid = H5Dcreate1(grpid, PRES_NAME, H5T_NATIVE_FLOAT,
                                    spaceid, cparmsid)) < 0) ERR;
-        if ((temp_dsid = H5Dcreate(grpid, TEMP_NAME, H5T_NATIVE_FLOAT,
+        if ((temp_dsid = H5Dcreate1(grpid, TEMP_NAME, H5T_NATIVE_FLOAT,
                                    spaceid, cparmsid)) < 0) ERR;
 
         /* Get the spaceid and check various things. */
@@ -153,8 +153,9 @@ main()
         start[0] = 1;
         start[1] = 0;
         start[2] = 0;
+        ones[0] = ones[1] = ones[2] = 1;
         if (H5Sselect_hyperslab(write_spaceid, H5S_SELECT_SET,
-                                start, NULL, count, NULL) < 0) ERR;
+                                start, NULL, ones, count) < 0) ERR;
 
         /* Write second record of data to each dataset. */
         if (H5Dwrite(pres_dsid, H5T_IEEE_F32LE, mem_spaceid, write_spaceid,
@@ -198,7 +199,7 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Write an array of bools, with compression. */
         dims[0] = DIM1_LEN;
@@ -207,7 +208,7 @@ main()
         if (H5Pset_chunk(propid, 1, dims)) ERR;
         if (H5Pset_deflate(propid, DEFLATE_LEVEL)) ERR;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+        if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                    spaceid, propid)) < 0) ERR;
         if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      bool_out) < 0) ERR;
@@ -220,7 +221,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
         if ((datasetid = H5Dopen1(grpid, VAR_BOOL_NAME)) < 0) ERR;
         if ((propid = H5Dget_create_plist(datasetid)) < 0) ERR;
 
@@ -251,12 +252,11 @@ main()
         size_t namelen = MAX_NAME;
         char name[MAX_NAME + 1];
         int found_shuffle = 0, found_fletcher32 = 0, found_deflate = 0;
-        int f;
 
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Write an array of bools, with compression, fletcher32
          * checksum, shuffle filters. Like a hoogie with "the works." */
@@ -268,7 +268,7 @@ main()
         if (H5Pset_deflate(propid, DEFLATE_LEVEL)) ERR;
         if (H5Pset_fletcher32(propid)) ERR;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((datasetid = H5Dcreate(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
+        if ((datasetid = H5Dcreate1(grpid, VAR_BOOL_NAME, H5T_NATIVE_HBOOL,
                                    spaceid, propid)) < 0) ERR;
         if (H5Dwrite(datasetid, H5T_NATIVE_HBOOL, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      bool_out) < 0) ERR;
@@ -281,7 +281,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
         if ((datasetid = H5Dopen1(grpid, VAR_BOOL_NAME)) < 0) ERR;
         if ((propid = H5Dget_create_plist(datasetid)) < 0) ERR;
 
@@ -289,7 +289,7 @@ main()
          * found in H5Zpublic.h. */
         if ((num_filters = H5Pget_nfilters(propid)) < 0) ERR;
         if (num_filters != 3) ERR;
-        for (f = 0; f < num_filters; f++)
+        for (unsigned int f = 0; f < num_filters; f++)
         {
             if ((filter = H5Pget_filter2(propid, f, &flags, &cd_nelems, cd_values,
                                          namelen, name, &filter_config)) < 0) ERR;
@@ -339,16 +339,16 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Create a dataset of native endian. */
         dims[0] = DIM1_LEN;
         if ((spaceid = H5Screate_simple(1, dims, dims)) < 0) ERR;
-        if ((native_did = H5Dcreate(grpid, NATIVE_VAR_NAME, H5T_NATIVE_INT,
+        if ((native_did = H5Dcreate1(grpid, NATIVE_VAR_NAME, H5T_NATIVE_INT,
                                     spaceid, H5P_DEFAULT)) < 0) ERR;
-        if ((le_did = H5Dcreate(grpid, LE_VAR_NAME, H5T_STD_I32LE,
+        if ((le_did = H5Dcreate1(grpid, LE_VAR_NAME, H5T_STD_I32LE,
                                 spaceid, H5P_DEFAULT)) < 0) ERR;
-        if ((be_did = H5Dcreate(grpid, BE_VAR_NAME, H5T_STD_I32BE,
+        if ((be_did = H5Dcreate1(grpid, BE_VAR_NAME, H5T_STD_I32BE,
                                 spaceid, H5P_DEFAULT)) < 0) ERR;
         if (H5Dwrite(native_did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
                      data) < 0) ERR;
@@ -366,7 +366,7 @@ main()
 
         /* Now reopen the file and check. */
         if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+        if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
         if ((native_did = H5Dopen1(grpid, NATIVE_VAR_NAME)) < 0) ERR;
         if ((typeid = H5Dget_type(native_did)) < 0) ERR;
@@ -409,7 +409,7 @@ main()
     }
 
     SUMMARIZE_ERR;
-#ifdef USE_SZIP
+#ifdef HAVE_H5Z_SZIP
     printf("*** Checking szip functionality...");
 #define SZIP_VAR_NAME "szip_var"
 #define SZIP_DIM1_LEN 32
@@ -432,7 +432,7 @@ main()
         /* Open file and create group. */
         if ((fileid = H5Fcreate(FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT,
                                 H5P_DEFAULT)) < 0) ERR;
-        if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+        if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
         /* Create dataset creation property list. */
         if ((plistid = H5Pcreate(H5P_DATASET_CREATE)) < 0) ERR;
@@ -474,7 +474,7 @@ main()
             int i;
 
             if ((fileid = H5Fopen(FILE_NAME, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-            if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+            if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
             if ((native_did = H5Dopen1(grpid, SZIP_VAR_NAME)) < 0) ERR;
 
@@ -511,16 +511,16 @@ main()
         /* Run test 4 times. */
         for (f = 0; f < NUM_FILE; f++)
         {
-            char file_name[MAX_STR + 1];
+            char file_name[MAX_STR * 2 + 1];
             char desc[NUM_FILE][MAX_STR + 1] = {"uncompressed", "zlib",
                                                 "szip", "zlib_and_szip",
                                                 "szip_and_zlib"};
 
             /* Open file and create group. */
-            sprintf(file_name, "%s_%s.h5", TEST_NAME, desc[f]);
+            snprintf(file_name, sizeof(file_name), "%s_%s.h5", TEST_NAME, desc[f]);
             if ((fileid = H5Fcreate(file_name, H5F_ACC_TRUNC, H5P_DEFAULT,
                                     H5P_DEFAULT)) < 0) ERR;
-            if ((grpid = H5Gcreate(fileid, GRP_NAME, 0)) < 0) ERR;
+            if ((grpid = H5Gcreate1(fileid, GRP_NAME, 0)) < 0) ERR;
 
             /* Create dataset creation property list. */
             if ((plistid = H5Pcreate(H5P_DATASET_CREATE)) < 0) ERR;
@@ -579,7 +579,7 @@ main()
                 int i;
 
                 if ((fileid = H5Fopen(file_name, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0) ERR;
-                if ((grpid = H5Gopen(fileid, GRP_NAME)) < 0) ERR;
+                if ((grpid = H5Gopen1(fileid, GRP_NAME)) < 0) ERR;
 
                 if ((native_did = H5Dopen1(grpid, BOTH_VAR_NAME)) < 0) ERR;
 
@@ -596,6 +596,6 @@ main()
         } /* next file */
     }
     SUMMARIZE_ERR;
-#endif /* USE_SZIP */
+#endif /* HAVE_H5Z_SZIP */
     FINAL_RESULTS;
 }
