@@ -1,4 +1,4 @@
-/* Copyright 2018-2018 University Corporation for Atmospheric
+/* Copyright 2018-2022 University Corporation for Atmospheric
    Research/Unidata. */
 /**
  * @file This header file contains macros, types, and prototypes for
@@ -60,12 +60,13 @@ struct NCauth;
 /** Struct to hold HDF5-specific info for the file. */
 typedef struct NC_HDF5_FILE_INFO {
    hid_t hdfid;
-#if defined(ENABLE_BYTERANGE) || defined(ENABLE_HDF5_ROS3) || defined(ENABLE_S3_SDK)
-   struct HTTP {
-	NCURI* uri; /* Parse of the incoming path, if url */
-	int iosp; /* We are using the S3 rawvirtual file driver */
-	struct NCauth* auth;
-   } http;
+   unsigned transientid; /* counter for transient ids */
+   NCURI* uri; /* Parse of the incoming path, if url */
+#if defined(NETCDF_ENABLE_BYTERANGE)
+   int byterange;
+#endif
+#ifdef NETCDF_ENABLE_S3
+   struct NCauth* auth;
 #endif
 } NC_HDF5_FILE_INFO_T;
 
@@ -182,6 +183,7 @@ int nc4_hdf5_get_chunk_cache(int ncid, size_t *sizep, size_t *nelemsp,
 int NC4_hdf5_def_var_filter(int ncid, int varid, unsigned int filterid, size_t nparams, const unsigned int *params);
 int NC4_hdf5_inq_var_filter_ids(int ncid, int varid, size_t* nfiltersp, unsigned int *filterids);
 int NC4_hdf5_inq_var_filter_info(int ncid, int varid, unsigned int filterid, size_t* nparamsp, unsigned int *params);
+int NC4_hdf5_inq_filter_avail(int ncid, unsigned id);
 
 /* Filterlist management */
 
@@ -194,12 +196,18 @@ struct NC_HDF5_Filter {
     unsigned int* params;  /**< Params for arbitrary filter. */
 };
 
+int NC4_hdf5_filter_initialize(void);
+int NC4_hdf5_filter_finalize(void);
 int NC4_hdf5_filter_remove(NC_VAR_INFO_T* var, unsigned int id);
 int NC4_hdf5_filter_lookup(NC_VAR_INFO_T* var, unsigned int id, struct NC_HDF5_Filter** fi);
 int NC4_hdf5_addfilter(NC_VAR_INFO_T* var, unsigned int id, size_t nparams, const unsigned int* params, int flags);
 int NC4_hdf5_filter_freelist(NC_VAR_INFO_T* var);
 int NC4_hdf5_find_missing_filter(NC_VAR_INFO_T* var, unsigned int* idp);
 
+/* Add an attribute to the attribute list. */
+int nc4_put_att(NC_GRP_INFO_T* grp, int varid, const char *name, nc_type file_type,
+		size_t len, const void *data, nc_type mem_type, int force);
+
 /* Support functions for provenance info (defined in nc4hdf.c) */
 extern int NC4_hdf5get_libversion(unsigned*,unsigned*,unsigned*);/*libsrc4/nc4hdf.c*/
 extern int NC4_hdf5get_superblock(struct NC_FILE_INFO*, int*);/*libsrc4/nc4hdf.c*/
@@ -210,4 +218,11 @@ extern int nc4_find_default_chunksizes2(NC_GRP_INFO_T *grp, NC_VAR_INFO_T *var);
 EXTERNL hid_t nc4_H5Fopen(const char *filename, unsigned flags, hid_t fapl_id);
 EXTERNL hid_t nc4_H5Fcreate(const char *filename, unsigned flags, hid_t fcpl_id, hid_t fapl_id);
 
+int hdf5set_format_compatibility(hid_t fapl_id);
+
+/* HDF5 initialization/finalization */
+extern int nc4_hdf5_initialized;
+extern void nc4_hdf5_initialize(void);
+extern void nc4_hdf5_finalize(void);
+
 #endif /* _HDF5INTERNAL_ */
