@@ -6,6 +6,7 @@
 #include "dapincludes.h"
 #include "daputil.h"
 #include "dapdump.h"
+#include <stddef.h>
 
 #ifdef DAPDEBUG
 extern char* ocfqn(OCddsnode);
@@ -15,8 +16,8 @@ extern char* ocfqn(OCddsnode);
 static NCerror sequencecheckr(CDFnode* node, NClist* vars, CDFnode* topseq);
 static NCerror restructr(NCDAPCOMMON*, CDFnode*, CDFnode*, NClist*);
 static NCerror repairgrids(NCDAPCOMMON*, NClist*);
-static NCerror structwrap(NCDAPCOMMON*, CDFnode*, CDFnode*, int, CDFnode*, int);
-static int findin(CDFnode* parent, CDFnode* child);
+static NCerror structwrap(NCDAPCOMMON*, CDFnode*, CDFnode*, size_t, CDFnode*);
+static size_t findin(CDFnode* parent, CDFnode* child);
 static CDFnode* makenewstruct(NCDAPCOMMON*, CDFnode*, CDFnode*);
 static NCerror mapnodesr(CDFnode*, CDFnode*, int depth);
 static NCerror mapfcn(CDFnode* dstnode, CDFnode* srcnode);
@@ -70,7 +71,7 @@ computecdfnodesets(NCDAPCOMMON* nccomm, CDFtree* tree)
 NCerror
 computevarnodes(NCDAPCOMMON* nccomm, NClist* allnodes, NClist* varnodes)
 {
-    unsigned int i,len;
+    size_t i, len;
     NClist* allvarnodes = nclistnew();
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(allnodes,i);
@@ -134,10 +135,10 @@ fprintf(stderr,"computevarnodes: var: %s\n",makecdfpathstring(node,"."));
 NCerror
 fixgrid(NCDAPCOMMON* nccomm, CDFnode* grid)
 {
-    unsigned int i,glen;
+    unsigned int i;
     CDFnode* array;
 
-    glen = nclistlength(grid->subnodes);
+    size_t glen = nclistlength(grid->subnodes);
     array = (CDFnode*)nclistget(grid->subnodes,0);
     if(nccomm->controls.flags & (NCF_NC3)) {
         /* Rename grid Array: variable, but leave its oc base name alone */
@@ -433,7 +434,7 @@ we expected a grid.
 static int
 restructr(NCDAPCOMMON* ncc, CDFnode* dxdparent, CDFnode* patternparent, NClist* repairlist)
 {
-    int index, i, j, match;
+    size_t index, i, j, match;
 
 #ifdef DEBUG
 fprintf(stderr,"restruct: matched: %s -> %s\n",
@@ -501,15 +502,14 @@ static NCerror
 repairgrids(NCDAPCOMMON* ncc, NClist* repairlist)
 {
     NCerror ncstat = NC_NOERR;
-    int i;
+    size_t i;
     assert(nclistlength(repairlist) % 2 == 0);
     for(i=0;i<nclistlength(repairlist);i+=2) {
 	CDFnode* node = (CDFnode*)nclistget(repairlist,i);
 	CDFnode* pattern = (CDFnode*)nclistget(repairlist,i+1);
-	int index = findin(node->container,node);
-	int tindex = findin(pattern->container,pattern);
+	size_t index = findin(node->container,node);
 	ncstat = structwrap(ncc, node,node->container,index,
-                             pattern->container,tindex);
+                             pattern->container);
 #ifdef DEBUG
 fprintf(stderr,"repairgrids: %s -> %s\n",
 ocfqn(node->ocnode),ocfqn(pattern->ocnode));
@@ -520,8 +520,8 @@ ocfqn(node->ocnode),ocfqn(pattern->ocnode));
 }
 
 static NCerror
-structwrap(NCDAPCOMMON* ncc, CDFnode* node, CDFnode* parent, int parentindex,
-                           CDFnode* patterngrid, int gridindex)
+structwrap(NCDAPCOMMON* ncc, CDFnode* node, CDFnode* parent, size_t parentindex,
+                           CDFnode* patterngrid)
 {
     CDFnode* newstruct;
 
@@ -538,12 +538,11 @@ structwrap(NCDAPCOMMON* ncc, CDFnode* node, CDFnode* parent, int parentindex,
     return NC_NOERR;
 }
 
-static int
+static size_t
 findin(CDFnode* parent, CDFnode* child)
 {
-    int i;
     NClist* subnodes = parent->subnodes;
-    for(i=0;i<nclistlength(subnodes);i++) {
+    for(size_t i=0;i<nclistlength(subnodes);i++) {
 	if(nclistget(subnodes,i) == child)
 	    return i;
     }
@@ -674,13 +673,13 @@ dimimprint(NCDAPCOMMON* nccomm)
 {
     NCerror ncstat = NC_NOERR;
     NClist* allnodes;
-    int i,j;
+    size_t i,j;
     CDFnode* basenode;
 
     allnodes = nccomm->cdf.ddsroot->tree->nodes;
     for(i=0;i<nclistlength(allnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(allnodes,i);
-	int noderank, baserank;
+	size_t noderank, baserank;
         /* Do dimension imprinting */
 	basenode = node->basenode;
 	if(basenode == NULL) continue;
@@ -689,7 +688,7 @@ dimimprint(NCDAPCOMMON* nccomm)
 	if(noderank == 0) continue;
         ASSERT(noderank == baserank);
 #ifdef DEBUG
-fprintf(stderr,"dimimprint %s/%d -> %s/%d\n",
+fprintf(stderr,"dimimprint %s/%zu -> %s/%zu\n",
 	makecdfpathstring(basenode,"."),
 	noderank,
 	makecdfpathstring(node,"."),
@@ -725,7 +724,7 @@ static NClist*
 clonedimset(NCDAPCOMMON* nccomm, NClist* dimset, CDFnode* var)
 {
     NClist* result = NULL;
-    int i;
+    size_t i;
 
     for(i=0;i<nclistlength(dimset);i++) {
         CDFnode *dim = NULL;
@@ -768,7 +767,7 @@ definedimsetplus(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 static NCerror
 definedimsetall(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 {
-    int i;
+    size_t i;
     int ncstat = NC_NOERR;
     NClist* dimsetall = NULL;
 
@@ -795,7 +794,7 @@ fprintf(stderr,"dimsetall: |%s|=%d\n",node->ocname,(int)nclistlength(dimsetall))
 static NCerror
 definetransdimset(NCDAPCOMMON* nccomm/*notused*/, CDFnode* node)
 {
-    int i;
+    size_t i;
     int ncstat = NC_NOERR;
     NClist* dimsettrans = NULL;
 
@@ -842,7 +841,7 @@ Recursive helper for definedimsettrans3
 static NCerror
 definedimsettransR(NCDAPCOMMON* nccomm, CDFnode* node)
 {
-    int i;
+    size_t i;
     int ncstat = NC_NOERR;
 
     definetransdimset(nccomm,node);
@@ -882,7 +881,7 @@ Recursive helper
 static NCerror
 definedimsetsR(NCDAPCOMMON* nccomm, CDFnode* node)
 {
-    int i;
+    size_t i;
     int ncstat = NC_NOERR;
 
     definedimsetplus(nccomm,node);
@@ -1057,7 +1056,7 @@ buildcdftreer(NCDAPCOMMON* nccomm, OCddsnode ocnode, CDFnode* container,
 void
 freecdfroot(CDFnode* root)
 {
-    int i;
+    size_t i;
     CDFtree* tree;
     NCDAPCOMMON* nccomm;
     if(root == NULL) return;
@@ -1187,7 +1186,7 @@ fix1node(NCDAPCOMMON* nccomm, CDFnode* node)
 static NCerror
 fixnodes(NCDAPCOMMON* nccomm, NClist* cdfnodes)
 {
-    int i;
+    size_t i;
     for(i=0;i<nclistlength(cdfnodes);i++) {
 	CDFnode* node = (CDFnode*)nclistget(cdfnodes,i);
 	NCerror err = fix1node(nccomm,node);
