@@ -5,9 +5,9 @@
 
 #include "d4includes.h"
 #include <stdarg.h>
+#include <stddef.h>
 #include "nc4internal.h"
 #include "ncoffsets.h"
-#include "ezxml.h"
 
 /**
  * Build the netcdf-4 metadata from the NCD4node nodes.
@@ -39,7 +39,7 @@ static int convertString(union ATOMICS* converter, NCD4node* type, const char* s
 static void* copyAtomic(union ATOMICS* converter, nc_type type, size_t len, void* dst, NClist* blobs);
 static int decodeEconst(NCD4meta* builder, NCD4node* enumtype, const char* nameorval, union ATOMICS* converter);
 static int downConvert(union ATOMICS* converter, NCD4node* type);
-static void freeStringMemory(char** mem, int count);
+static void freeStringMemory(char** mem, size_t count);
 static size_t getDimrefs(NCD4node* var, int* dimids);
 static size_t getDimsizes(NCD4node* var, int* dimsizes);
 static d4size_t getpadding(d4size_t offset, size_t alignment);
@@ -47,6 +47,7 @@ static int markdapsize(NCD4meta* meta);
 static int markfixedsize(NCD4meta* meta);
 static void savegroupbyid(NCD4meta*,NCD4node* group);
 static void savevarbyid(NCD4node* group, NCD4node* var);
+static int isfilltypecompatible(NCD4node* parenttype, NCD4node* attrtype);
 #ifndef FIXEDOPAQUE
 static int buildBytestringType(NCD4meta* builder);
 #endif
@@ -58,20 +59,21 @@ int
 NCD4_metabuild(NCD4meta* metadata, int ncid)
 {
     int ret = NC_NOERR;
-    int i;
+    size_t i;
 
     metadata->ncid = ncid;
     metadata->root->meta.id = ncid;
 
     /* Fix up the atomic types */
-    for(i=0;i<nclistlength(metadata->allnodes);i++) {
-	NCD4node* n = (NCD4node*)nclistget(metadata->allnodes,i);
+    for(i=0;i<nclistlength(metadata->atomictypes);i++) {
+	NCD4node* n = (NCD4node*)nclistget(metadata->atomictypes,i);
 	if(n->sort != NCD4_TYPE) continue;
 	if(n->subsort > NC_MAX_ATOMIC_TYPE) continue;
 	n->meta.id = n->subsort;
         n->meta.isfixedsize = (n->subsort == NC_STRING ? 0 : 1);
 	if(n->subsort <= NC_STRING)
 	    n->meta.dapsize = NCD4_typesize(n->subsort);
+	n->container = metadata->root;
     }
 
     /* Topo sort the set of all nodes */
@@ -88,79 +90,27 @@ done:
     return THROW(ret);
 }
 
-
-/* Create an empty NCD4meta object for
-   use in subsequent calls
-   (is the the right src file to hold this?)
-*/
-
-NCD4meta*
-NCD4_newmeta(size_t rawsize, void* rawdata)
-{
-    NCD4meta* meta = (NCD4meta*)calloc(1,sizeof(NCD4meta));
-    if(meta == NULL) return NULL;
-    meta->allnodes = nclistnew();
-    meta->serial.rawsize = rawsize;
-    meta->serial.rawdata = rawdata;
-#ifdef D4DEBUG
-    meta->debuglevel = 1;
-#endif
-    return meta;
-}
-
-void
-NCD4_setdebuglevel(NCD4meta* meta, int debuglevel)
-{
-    meta->debuglevel = debuglevel;
-}
-
-void
-NCD4_reclaimMeta(NCD4meta* dataset)
-{
-    int i;
-    if(dataset == NULL) return;
-    for(i=0;i<nclistlength(dataset->allnodes);i++) {
-	NCD4node* node = (NCD4node*)nclistget(dataset->allnodes,i);
-	reclaimNode(node);
-    }
-    nullfree(dataset->error.parseerror);
-    nullfree(dataset->error.message);
-    nullfree(dataset->error.context);
-    nullfree(dataset->error.otherinfo);
-    nullfree(dataset->serial.errdata);
-    nclistfree(dataset->groupbyid);
-#if 0
-    for(i=0;i<nclistlength(dataset->blobs);i++) {
-	void* p = nclistget(dataset->blobs,i);
-	nullfree(p);
-    }
-    nclistfree(dataset->blobs);
-#endif
-    nclistfree(dataset->allnodes);
-    nullfree(dataset->serial.dmr);
-    free(dataset);
-}
-
 void
 reclaimNode(NCD4node* node)
 {
     if(node == NULL) return;
-    nullfree(node->name);
-    nclistfree(node->groups);
-    nclistfree(node->vars);
-    nclistfree(node->types);
-    nclistfree(node->dims);
-    nclistfree(node->attributes);
-    nclistfree(node->maps);
-    nclistfreeall(node->xmlattributes);
-    nclistfreeall(node->attr.values);
-    nclistfree(node->en.econsts);
-    nclistfree(node->group.elements);
-    nullfree(node->group.dapversion);
-    nullfree(node->group.dmrversion);
-    nullfree(node->group.datasetname);
-    nclistfree(node->group.varbyid);
-    nullfree(node->nc4.orig.name);
+    nullfree(node->name); node->name = NULL;
+    nclistfree(node->groups); node->groups = NULL;
+    nclistfree(node->vars); node->vars = NULL;
+    nclistfree(node->types); node->types = NULL;
+    nclistfree(node->dims); node->dims = NULL;
+    nclistfree(node->attributes); node->attributes = NULL;
+    nclistfreeall(node->mapnames); node->mapnames = NULL;
+    nclistfree(node->maps); node->maps = NULL;
+    nclistfreeall(node->xmlattributes); node->xmlattributes = NULL;
+    nclistfreeall(node->attr.values); node->attr.values = NULL;
+    nclistfree(node->en.econsts); node->en.econsts = NULL;
+    nclistfree(node->group.elements); node->group.elements = NULL;
+    nullfree(node->group.dapversion); node->group.dapversion = NULL;
+    nullfree(node->group.dmrversion); node->group.dmrversion = NULL;
+    nullfree(node->group.datasetname); node->group.datasetname = NULL;
+    nclistfree(node->group.varbyid); node->group.varbyid = NULL;
+    nullfree(node->nc4.orig.name); node->nc4.orig.name = NULL;
     nullfree(node);
 }
 
@@ -170,7 +120,8 @@ reclaimNode(NCD4node* node)
 static int
 build(NCD4meta* builder, NCD4node* root)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
     size_t len = nclistlength(builder->allnodes);
 
     /* Tag the root group */
@@ -274,7 +225,8 @@ done:
 static int
 buildGroups(NCD4meta* builder, NCD4node* parent)
 {
-    int i,ret=NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
 #ifdef D4DEBUG
     fprintf(stderr,"build group: %s\n",parent->name);
 #endif
@@ -303,7 +255,7 @@ buildDimension(NCD4meta* builder, NCD4node* dim)
     if(dim->dim.isunlimited) {
 	NCCHECK((nc_def_dim(group->meta.id,dim->name,NC_UNLIMITED,&dim->meta.id)));
     } else {
-	NCCHECK((nc_def_dim(group->meta.id,dim->name,(size_t)dim->dim.size,&dim->meta.id)));
+	NCCHECK((nc_def_dim(group->meta.id,dim->name,dim->dim.size,&dim->meta.id)));
     }
 done:
     return THROW(ret);
@@ -312,7 +264,8 @@ done:
 static int
 buildEnumeration(NCD4meta* builder, NCD4node* en)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
     NCD4node* group = NCD4_groupFor(en);
     NCCHECK((nc_def_enum(group->meta.id,en->basetype->meta.id,en->name,&en->meta.id)));
     for(i=0;i<nclistlength(en->en.econsts);i++) {
@@ -393,7 +346,8 @@ done:
 static int
 buildMaps(NCD4meta* builder, NCD4node* var)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
     size_t count = nclistlength(var->maps);
     char** memory = NULL;
     char** p;
@@ -423,7 +377,8 @@ done:
 static int
 buildAttributes(NCD4meta* builder, NCD4node* varorgroup)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
     NClist* blobs = NULL;
 
     for(i=0;i<nclistlength(varorgroup->attributes);i++) {
@@ -437,6 +392,10 @@ buildAttributes(NCD4meta* builder, NCD4node* varorgroup)
 	if(strncmp(attr->name,UCARTAG,strlen(UCARTAG)) == 0)
 	    continue;
 
+	/* Suppress all reserved attributes */
+	if(NCD4_lookupreserved(attr->name) != NULL)
+	    continue;
+
 	if(ISGROUP(varorgroup->sort))
 	    varid = NC_GLOBAL;
 	else
@@ -446,9 +405,11 @@ buildAttributes(NCD4meta* builder, NCD4node* varorgroup)
 	    nullfree(memory);
             FAIL(ret,"Malformed attribute value(s) for: %s",attr->name);
         }
-	group = NCD4_groupFor(varorgroup);
-        NCCHECK((nc_put_att(group->meta.id,varid,attr->name,attr->basetype->meta.id,count,memory)));
-	nclistfreeall(blobs); blobs = NULL;
+        if(memory != NULL) {
+  	    group = NCD4_groupFor(varorgroup);
+            NCCHECK((nc_put_att(group->meta.id,varid,attr->name,attr->basetype->meta.id,count,memory)));
+	}
+        nclistfreeall(blobs); blobs = NULL;
         nullfree(memory);
     }
 done:
@@ -524,7 +485,8 @@ done:
 static int
 buildCompound(NCD4meta* builder, NCD4node* cmpdtype, NCD4node* group, char* name)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
 
     /* Step 1: compute field offsets */
     computeOffsets(builder,cmpdtype);
@@ -534,15 +496,14 @@ buildCompound(NCD4meta* builder, NCD4node* cmpdtype, NCD4node* group, char* name
 
     /* Step 3: add the fields to type */
     for(i=0;i<nclistlength(cmpdtype->vars);i++) {
-	int rank;
 	int dimsizes[NC_MAX_VAR_DIMS];
         NCD4node* field = (NCD4node*)nclistget(cmpdtype->vars,i);
-	rank = nclistlength(field->dims);
+	size_t rank = nclistlength(field->dims);
         if(rank == 0) { /* scalar */
             NCCHECK((nc_insert_compound(group->meta.id, cmpdtype->meta.id,
 					field->name, field->meta.offset,
 					field->basetype->meta.id)));
-        } else if(rank > 0) { /* array  */
+        } else { /* array  */
   	    int idimsizes[NC_MAX_VAR_DIMS];
 	    int j;
 	    getDimsizes(field,dimsizes);
@@ -551,7 +512,7 @@ buildCompound(NCD4meta* builder, NCD4node* cmpdtype, NCD4node* group, char* name
             NCCHECK((nc_insert_array_compound(group->meta.id, cmpdtype->meta.id,
 					      field->name, field->meta.offset,
 					      field->basetype->meta.id,
-					      rank, idimsizes)));
+					      (int)rank, idimsizes)));
 	}
     }
 
@@ -574,7 +535,7 @@ buildAtomicVar(NCD4meta* builder, NCD4node* var)
 #endif
 
     rank = getDimrefs(var,dimids);
-    NCCHECK((nc_def_var(group->meta.id,var->name,var->basetype->meta.id,rank,dimids,&var->meta.id)));
+    NCCHECK((nc_def_var(group->meta.id,var->name,var->basetype->meta.id,(int)rank,dimids,&var->meta.id)));
     /* Tag the var */
     savevarbyid(group,var);
 
@@ -589,14 +550,13 @@ buildStructure(NCD4meta* builder, NCD4node* structvar)
 {
     int ret = NC_NOERR;
     NCD4node* group;
-    int rank;
     int dimids[NC_MAX_VAR_DIMS];
 
     /* Step 1: define the variable */
-    rank = nclistlength(structvar->dims);
+    size_t rank = nclistlength(structvar->dims);
     getDimrefs(structvar,dimids);
     group = NCD4_groupFor(structvar);
-    NCCHECK((nc_def_var(group->meta.id,structvar->name,structvar->basetype->meta.id,rank,dimids,&structvar->meta.id)));
+    NCCHECK((nc_def_var(group->meta.id,structvar->name,structvar->basetype->meta.id,(int)rank,dimids,&structvar->meta.id)));
     /* Tag the var */
     savevarbyid(group,structvar);
 
@@ -613,13 +573,12 @@ buildSequence(NCD4meta* builder, NCD4node* seq)
 
     int ret = NC_NOERR;
     NCD4node* group;
-    int rank;
     int dimids[NC_MAX_VAR_DIMS];
 
-    rank = nclistlength(seq->dims);
+    size_t rank = nclistlength(seq->dims);
     getDimrefs(seq,dimids);
     group = NCD4_groupFor(seq);
-    NCCHECK((nc_def_var(group->meta.id,seq->name,seq->basetype->meta.id,rank,dimids,&seq->meta.id)));
+    NCCHECK((nc_def_var(group->meta.id,seq->name,seq->basetype->meta.id,(int)rank,dimids,&seq->meta.id)));
     savevarbyid(group,seq);
 
     /* Build attributes and map attributes WRT the variable */
@@ -648,8 +607,41 @@ savevarbyid(NCD4node* group, NCD4node* var)
 {
     if(group->group.varbyid == NULL)
         group->group.varbyid = nclistnew();
-    nclistsetalloc(group->group.varbyid,var->meta.id);
-    nclistinsert(group->group.varbyid,var->meta.id,var);
+    nclistsetalloc(group->group.varbyid, (size_t)var->meta.id);
+    nclistinsert(group->group.varbyid, (size_t)var->meta.id,var);
+}
+
+/* Collect FQN path from var node up to and including
+   the root group and create an name from it
+*/
+char*
+NCD4_getVarFQN(NCD4node* var, const char* tail)
+{
+    size_t i;
+    NCD4node* x = NULL;
+    NClist* path = NULL;
+    NCbytes* fqn =  NULL;
+    char* result;
+
+    path = nclistnew();
+    for(x=var->container;ISGROUP(x->sort);x=x->container) {
+	nclistinsert(path,0,x);
+    }
+    fqn = ncbytesnew();
+    for(i=0;i<nclistlength(path);i++) {
+	NCD4node* grp = (NCD4node*)nclistget(path,i);
+	char* escaped = backslashEscape(grp->name);
+	if(escaped == NULL) return NULL;
+	if(i > 0) ncbytesappend(fqn,'/');
+	ncbytescat(fqn,escaped);
+	free(escaped);
+    }
+    nclistfree(path);
+    if(tail != NULL)
+        ncbytescat(fqn,tail);
+    result = ncbytesextract(fqn);
+    ncbytesfree(fqn);
+    return result;
 }
 
 /* Collect FQN path from node up to (but not including)
@@ -658,7 +650,7 @@ savevarbyid(NCD4node* group, NCD4node* var)
 static char*
 getFieldFQN(NCD4node* field, const char* tail)
 {
-    int i;
+    size_t i;
     NCD4node* x = NULL;
     NClist* path = NULL;
     NCbytes* fqn =  NULL;
@@ -688,8 +680,8 @@ getFieldFQN(NCD4node* field, const char* tail)
 static size_t
 getDimrefs(NCD4node* var, int* dimids)
 {
-    int i;
-    int rank = nclistlength(var->dims);
+    size_t i;
+    size_t rank = nclistlength(var->dims);
     for(i=0;i<rank;i++) {
 	NCD4node* dim = (NCD4node*)nclistget(var->dims,i);
 	dimids[i] = dim->meta.id;
@@ -700,8 +692,8 @@ getDimrefs(NCD4node* var, int* dimids)
 static size_t
 getDimsizes(NCD4node* var, int* dimsizes)
 {
-    int i;
-    int rank = nclistlength(var->dims);
+    size_t i;
+    size_t rank = nclistlength(var->dims);
     for(i=0;i<rank;i++) {
 	NCD4node* dim = (NCD4node*)nclistget(var->dims,i);
 	dimsizes[i] = (int)dim->dim.size;
@@ -713,11 +705,10 @@ getDimsizes(NCD4node* var, int* dimsizes)
 /* Utilities */
 
 static void
-freeStringMemory(char** mem, int count)
+freeStringMemory(char** mem, size_t count)
 {
-    int i;
     if(mem == NULL) return;
-    for(i=0;i<count;i++) {
+    for(size_t i=0;i<count;i++) {
 	char* p = mem[i];
         if(p) free(p);
     }
@@ -732,7 +723,8 @@ to nc_put_att().
 static int
 compileAttrValues(NCD4meta* builder, NCD4node* attr, void** memoryp, NClist* blobs)
 {
-    int i,ret = NC_NOERR;
+    size_t i;
+    int ret = NC_NOERR;
     unsigned char* memory = NULL;
     unsigned char* p;
     size_t size;
@@ -742,23 +734,25 @@ compileAttrValues(NCD4meta* builder, NCD4node* attr, void** memoryp, NClist* blo
     NCD4node* container = attr->container;
     NCD4node* basetype = attr->basetype;
     NClist* values = attr->attr.values;
-    int count = nclistlength(values);
+    size_t count = nclistlength(values);
 
     memset((void*)&converter,0,sizeof(converter));
 
     /* Deal with _FillValue */
     if(container->sort == NCD4_VAR && strcmp(attr->name,"_FillValue")==0) {
-	/* Verify or fix or fail on type match */
-	if(container->basetype != basetype) {
-	    /* _FillValue/Variable type mismatch */
-	    if(FLAGSET(builder->controller->controls.flags,NCF_FILLMISMATCH)) {
+	/* Verify or fix or ignore or fail on type mismatch */
+	if(container->basetype != basetype) {/* _FillValue/Variable type mismatch */
+	    int compatible = isfilltypecompatible(container->basetype, basetype);
+	    /* Test type compatibility */
+	    if(compatible && FLAGSET(builder->controller->controls.flags,NCF_FILLMISMATCH)) {
 		/* Force type match */
 		basetype = (attr->basetype = container->basetype);
-	    } else {/* Fail */
+	    } else if(FLAGSET(builder->controller->controls.flags,NCF_FILLMISMATCH_FAIL)) {/* Fail */
 	        FAIL(NC_EBADTYPE,"_FillValue/Variable type mismatch: %s:%s",container->name,attr->name);
-	    }
+	    } else
+	        goto done; /* ignore */
 	}
-    }
+    }  
     isenum = (basetype->subsort == NC_ENUM);
     truebase = (isenum ? basetype->basetype : basetype);
     if(!ISTYPE(truebase->sort) || (truebase->meta.id > NC_MAX_ATOMIC_TYPE))
@@ -897,8 +891,9 @@ Note: this will work if the econst string is a number or a econst name
 static int
 decodeEconst(NCD4meta* builder, NCD4node* enumtype, const char* nameorval, union ATOMICS* converter)
 {
-    int i,ret=NC_NOERR;
-    union ATOMICS number;
+    size_t i;
+    int ret = NC_NOERR;
+    union ATOMICS number = {0};
     NCD4node* match = NULL;
 
     /* First, see if the value is an econst name */
@@ -907,7 +902,7 @@ decodeEconst(NCD4meta* builder, NCD4node* enumtype, const char* nameorval, union
         if(strcmp(ec->name,nameorval)==0) {match = ec; break;}
     }
     /* If no match, try to invert as a number to see if there is a matching econst */
-    if(!match) {
+    if(!match && enumtype->en.econsts) {
         /* get the incoming value as number */
         if((ret=convertString(&number,enumtype->basetype,nameorval)))
             goto done;
@@ -959,7 +954,7 @@ backslashEscape(const char* s)
 static int
 markfixedsize(NCD4meta* meta)
 {
-    int i,j;
+    size_t i,j;
     for(i=0;i<nclistlength(meta->allnodes);i++) {
 	int fixed = 1;
 	NCD4node* n = (NCD4node*)nclistget(meta->allnodes,i);
@@ -989,7 +984,7 @@ markfixedsize(NCD4meta* meta)
 static void
 computeOffsets(NCD4meta* builder, NCD4node* cmpd)
 {
-    int i;
+    size_t i;
     d4size_t offset = 0;
     d4size_t largestalign = 1;
     d4size_t size = 0;
@@ -1113,7 +1108,7 @@ getpadding(d4size_t offset, size_t alignment)
 static int
 markdapsize(NCD4meta* meta)
 {
-    int i,j;
+    size_t i,j;
     for(i=0;i<nclistlength(meta->allnodes);i++) {
 	NCD4node* type = (NCD4node*)nclistget(meta->allnodes,i);
 	size_t totalsize;
@@ -1152,3 +1147,63 @@ markdapsize(NCD4meta* meta)
     }
     return NC_NOERR;
 }
+
+int
+NCD4_findvar(NC* ncp, int gid, int varid, NCD4node** varp, NCD4node** grpp)
+{
+    int ret = NC_NOERR;
+    NCD4INFO* info = NULL;
+    NCD4meta* meta = NULL;
+    NCD4node* var = NULL;
+    NCD4node* group = NULL;
+    int grp_id;
+
+    info = getdap(ncp);
+    if(info == NULL)
+	return THROW(NC_EBADID);
+    meta = info->dmrmetadata;
+    if(meta == NULL)
+	return THROW(NC_EBADID);
+    /* Locate var node via (grpid,varid) */
+    grp_id = GROUPIDPART(gid);
+    group = nclistget(meta->groupbyid, (size_t)grp_id);
+    if(group == NULL)
+	return THROW(NC_EBADID);
+    var = nclistget(group->group.varbyid, (size_t)varid);
+    if(var == NULL)
+	return THROW(NC_EBADID);
+    if(varp) *varp = var;
+    if(grpp) *grpp = group;
+    return THROW(ret);
+}
+
+static int
+isnumerictype(int typesort)
+{
+    switch (typesort) {
+    case NC_BYTE:
+    case NC_SHORT:
+    case NC_INT:
+    case NC_FLOAT:
+    case NC_DOUBLE:
+    case NC_UBYTE:
+    case NC_USHORT:
+    case NC_UINT:
+    case NC_INT64:
+    case NC_UINT64:
+	return 1;	
+    default: break;
+    }
+    return 0;
+}
+
+static int
+isfilltypecompatible(NCD4node* parenttype, NCD4node* attrtype)
+{
+    assert(parenttype->sort == NCD4_TYPE && attrtype->sort == NCD4_TYPE);
+    if(isnumerictype(parenttype->subsort) && isnumerictype(attrtype->subsort))
+        return 1;
+    if(parenttype->subsort == NC_ENUM && attrtype->subsort == NC_STRING)
+        return 1;
+    return 0;
+}
