@@ -3,6 +3,7 @@
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
  *********************************************************************/
 
+#include <stddef.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -11,6 +12,7 @@
 #include <direct.h>
 #endif
 #include "ncd4dispatch.h"
+#include "nc4internal.h"
 #include "d4includes.h"
 #include "d4curlfunctions.h"
 
@@ -34,8 +36,23 @@ static const NC_Dispatch NCD4_dispatch_base;
 
 const NC_Dispatch* NCD4_dispatch_table = NULL; /* moved here from ddispatch.c */
 
+/**************************************************/
+/* Define a structure defining reserved attributes */
+
+/** @internal List of reserved attributes. */
+static const NC_reservedatt NCD4_reserved[] = {
+    {D4CHECKSUMATTR, READONLYFLAG|NAMEONLYFLAG},      /*_DAP4_Checksum_CRC32*/
+    {D4LEATTR, READONLYFLAG|NAMEONLYFLAG},            /*_DAP4_Little_Endian*/
+    /* Also need to include the provenance attributes */
+    {NCPROPS, READONLYFLAG|NAMEONLYFLAG},		/*_NCProperties*/
+    {NULL, 0}
+};
+
 /* Forward */
 static int globalinit(void);
+static int ncd4_get_att_reserved(NC* ncp, int ncid, int varid, const char* name, void* value, nc_type t, const NC_reservedatt* rsvp);
+static int ncd4_inq_att_reserved(NC* ncp, int ncid, int varid, const char* name, nc_type* xtypep, size_t* lenp, const NC_reservedatt* rsvp);
+
 
 /**************************************************/
 int
@@ -46,14 +63,11 @@ NCD4_initialize(void)
     ncloginit();
 #ifdef D4DEBUG
     /* force logging to go to stderr */
-    nclogclose();
     if(nclogopen(NULL))
         ncsetlogging(1); /* turn it on */
 #endif
     /* Init global state */
     globalinit();
-    /* Load rc file */
-    NC_rcload();    
 
     return THROW(NC_NOERR);
 }
@@ -241,14 +255,6 @@ NCD4_set_var_chunk_cache(int ncid, int p2, size_t p3, size_t p4, float p5)
     return (NC_EPERM);
 }
 
-struct NC_FILTER_ACTION;
-
-static int
-NCD4_filter_actions(int ncid, int varid, int action, struct NC_Filterobject* spec)
-{
-    return (NC_EPERM);
-}
-
 /**************************************************/
 /*
 Following functions basically return the netcdf-4 value WRT to the nc4id.
@@ -334,19 +340,41 @@ NCD4_inq_att(int ncid, int varid, const char* name,
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+    
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
     substrateid = makenc4id(ncp,ncid);
+
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return ncd4_inq_att_reserved(ncp,ncid,varid,name,xtypep,lenp, rsvp);
     ret = nc_inq_att(substrateid, varid, name, xtypep, lenp);
     return (ret);
 }
 
+const struct NC_reservedatt*
+NCD4_lookupreserved(const char* name)
+{
+    const NC_reservedatt* p = NCD4_reserved;
+    for(;p->name;p++) {
+	if(strcmp(name,p->name)==0)
+	    return p;
+    }
+    return NULL;
+}
+
 static int
 NCD4_inq_attid(int ncid, int varid, const char *name, int *idp)
 {
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return NC_EATTMETA;
     substrateid = makenc4id(ncp,ncid);
     ret = nc_inq_attid(substrateid, varid, name, idp);
     return (ret);
@@ -358,9 +386,14 @@ NCD4_inq_attname(int ncid, int varid, int attnum, char* name)
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+    
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
     substrateid = makenc4id(ncp,ncid);
     ret = nc_inq_attname(substrateid, varid, attnum, name);
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return NC_EATTMETA;
     return (ret);
 }
 
@@ -370,21 +403,31 @@ NCD4_rename_att(int ncid, int varid, const char* name, const char* newname)
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+    
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return NC_EATTMETA;
     substrateid = makenc4id(ncp,ncid);
     ret = nc_rename_att(substrateid, varid, name, newname);
     return (ret);
 }
 
 static int
-NCD4_del_att(int ncid, int varid, const char* p3)
+NCD4_del_att(int ncid, int varid, const char* name)
 {
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+    
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return NC_EATTMETA;
     substrateid = makenc4id(ncp,ncid);
-    ret = nc_del_att(substrateid, varid, p3);
+    ret = nc_del_att(substrateid, varid, name);
     return (ret);
 }
 
@@ -394,7 +437,12 @@ NCD4_get_att(int ncid, int varid, const char* name, void* value, nc_type t)
     NC* ncp;
     int ret;
     int substrateid;
+    const NC_reservedatt* rsvp = NULL;
+    
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    /* Is this a reserved attribute name? */
+    if(name && (rsvp = NCD4_lookupreserved(name)))
+	return ncd4_get_att_reserved(ncp,ncid,varid,name,value,t,rsvp);
     substrateid = makenc4id(ncp,ncid);
     ret = NCDISPATCH_get_att(substrateid, varid, name, value, t);
     return (ret);
@@ -715,6 +763,54 @@ NCD4_get_var_chunk_cache(int ncid, int p2, size_t* p3, size_t* p4, float* p5)
     return (ret);
 }
 
+static int
+NCD4_inq_var_quantize(int ncid, int varid, int *quantize_modep, int *nsdp)
+{
+    NC* ncp;
+    int ret;
+    int substrateid;
+    if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    substrateid = makenc4id(ncp,ncid);
+    ret = nc_inq_var_quantize(substrateid, varid, quantize_modep, nsdp);
+    return (ret);
+}
+
+static int
+NCD4_inq_var_filter_ids(int ncid, int varid, size_t* nfilters, unsigned int* filterids)
+{
+    NC* ncp;
+    int ret;
+    int substrateid;
+    if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    substrateid = makenc4id(ncp,ncid);
+    ret = nc_inq_var_filter_ids(substrateid, varid, nfilters, filterids);
+    return (ret);
+}
+
+static int
+NCD4_inq_var_filter_info(int ncid, int varid, unsigned int id, size_t* nparams, unsigned int* params)
+{
+    NC* ncp;
+    int ret;
+    int substrateid;
+    if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    substrateid = makenc4id(ncp,ncid);
+    ret = nc_inq_var_filter_info(substrateid, varid, id, nparams, params);
+    return (ret);
+}
+
+static int
+NCD4_inq_filter_avail(int ncid, unsigned id)
+{
+    NC* ncp;
+    int ret;
+    int substrateid;
+    if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR) return (ret);
+    substrateid = makenc4id(ncp,ncid);
+    ret = nc_inq_filter_avail(substrateid, id);
+    return (ret);
+}
+
 /**************************************************/
 /*
 Following functions are overridden to handle 
@@ -746,13 +842,13 @@ NCD4_inq_dim(int ncid, int dimid, char* name, size_t* lenp)
     NC* ncp;
     NCD4INFO* info;
     NCD4meta* meta;
-    int i;
+    size_t i;
     NCD4node* dim = NULL;
 
     if((ret = NC_check_id(ncid, (NC**)&ncp)) != NC_NOERR)
 	goto done;
     info = (NCD4INFO*)ncp->dispatchdata;
-    meta = info->substrate.metadata;
+    meta = info->dmrmetadata;
 
     /* Locate the dimension specified by dimid */
     for(i=0;i<nclistlength(meta->allnodes);i++) {
@@ -772,13 +868,64 @@ done:
     return (ret);
 }
 
+static int
+ncd4_get_att_reserved(NC* ncp, int ncid, int varid, const char* name, void* value, nc_type t, const NC_reservedatt* rsvp)
+{
+    int ret = NC_NOERR;
+    NCD4node* var = NULL;
+
+    if((ret=NCD4_findvar(ncp,ncid,varid,&var,NULL))) goto done;
+
+    if(strcmp(rsvp->name,D4CHECKSUMATTR)==0) {
+	unsigned int* ip = (unsigned int*)value;
+	if(varid == NC_GLOBAL)
+            {ret = NC_EBADID; goto done;}
+	if(t != NC_UINT) {ret = NC_EBADTYPE; goto done;}
+	if(var->data.checksumattr == 0)
+	    {ret = NC_ENOTATT; goto done;} 
+	*ip = (var->data.remotechecksum);
+    } else if(strcmp(rsvp->name,D4LEATTR)==0) {
+	int* ip = (int*)value;
+	if(varid != NC_GLOBAL)
+            {ret = NC_EBADID; goto done;}
+	if(t != NC_INT) {ret = NC_EBADTYPE; goto done;}
+	*ip = (var->data.response->remotelittleendian?1:0);
+    }
+done:
+    return THROW(ret);
+}
+
+static int
+ncd4_inq_att_reserved(NC* ncp, int ncid, int varid, const char* name, nc_type* xtypep, size_t* lenp, const NC_reservedatt* rsvp)
+{
+    int ret = NC_NOERR;
+    NCD4node* var = NULL;
+
+    if(strcmp(rsvp->name,D4CHECKSUMATTR)==0) {
+	if(varid == NC_GLOBAL)
+            {ret = NC_EBADID; goto done;}
+        if((ret=NCD4_findvar(ncp,ncid,varid,&var,NULL))) goto done;
+	if(var->data.checksumattr == 0)
+	    {ret = NC_ENOTATT; goto done;} 
+	if(xtypep) *xtypep = NC_UINT;
+	if(lenp) *lenp = 1;
+    } else if(strcmp(rsvp->name,D4LEATTR)==0) {
+	if(varid != NC_GLOBAL)
+            {ret = NC_EBADID; goto done;}
+	if(xtypep) *xtypep = NC_INT;
+	if(lenp) *lenp = 1;
+    }
+done:
+    return THROW(ret);
+}
+
 /**************************************************/
 
 static int
 globalinit(void)
 {
     int stat = NC_NOERR;
-    return stat;
+    return THROW(stat);
 }
 
 /**************************************************/
@@ -870,7 +1017,11 @@ NCD4_def_var_filter,
 NCD4_set_var_chunk_cache,
 NCD4_get_var_chunk_cache,
 
-NCD4_filter_actions,
+NCD4_inq_var_filter_ids,
+NCD4_inq_var_filter_info,
+
+NC_NOTNC4_def_var_quantize,
+NCD4_inq_var_quantize,
+
+NCD4_inq_filter_avail,
 };
-
-
