@@ -15,8 +15,21 @@
 #include <wchar.h>
 #include <locale.h>
 //#include <direct.h>
+#if defined(DLL_NETCDF) /* define when library is a DLL */
+#  if defined(DLL_EXPORT) /* define when building the library */
+#   define MSC_EXTRA __declspec(dllexport)
+#  else
+#   define MSC_EXTRA __declspec(dllimport)
+#  endif
+#else
+#  define MSC_EXTRA
+#endif	/* defined(DLL_NETCDF) */
+#ifndef EXTERNL
+# define EXTERNL MSC_EXTRA extern
+#endif
 
-__declspec(dllexport)
+
+EXTERNL
 int
 getmountpoint(char* keyvalue, size_t size)
 {
