@@ -69,6 +69,8 @@ hdf4_rec_grp_del(NC_GRP_INFO_T *grp)
          * scale. */
         if (hdf4_var->sdsid && SDendaccess(hdf4_var->sdsid) < 0)
             return NC_EHDFERR;
+
+        nullfree(hdf4_var);
     }
 
     return NC_NOERR;
@@ -241,7 +243,6 @@ nc4_set_var_type(nc_type xtype, int endianness, size_t type_size, char *type_nam
     type->endianness = endianness;
     type->size = type_size;
     type->hdr.id = (size_t)xtype;
-    type->hdr.hashkey = NC_hashmapkey(type->hdr.name, strlen(type->hdr.name));
 
     /* Return to caller. */
     *typep = type;
@@ -423,6 +424,8 @@ nc4_var_list_add_full(NC_GRP_INFO_T* grp, const char* name, int ndims, nc_type x
     if ((retval = nc4_set_var_type(xtype, endianness, type_size, type_name,
                                    &(*var)->type_info)))
         return retval;
+    /* Propate the endianness to the variable */
+    (*var)->endianness = (*var)->type_info->endianness;
 
     (*var)->type_info->rc++;
 
@@ -482,7 +485,7 @@ hdf4_read_var(NC_FILE_INFO_T *h5, int v)
     int32 sdsid;
     int contiguous;
     int d, a;
-    int flag;
+    int32 flag;
     char name[NC_MAX_HDF4_NAME+1];
     int xtype;
     char type_name[NC_MAX_NAME + 1];
