@@ -19,12 +19,6 @@
 /*Mnemonic*/
 #define ENCODING 1
 
-/* list of environment variables to check for plugin roots */
-#define PLUGIN_ENV "HDF5_PLUGIN_PATH"
-#define PLUGIN_DIR_UNIX "/usr/local/hdf5/plugin"
-#define PLUGIN_DIR_WIN "%s/hdf5/lib/plugin"
-#define WIN32_ROOT_ENV "ALLUSERSPROFILE"
-
 /* Opaque */
 struct NCZ_Filter;
 
