@@ -3,6 +3,7 @@
  *      See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
 
+#include <stddef.h>
 #undef DEBUG
 
 /* Not sure this has any effect */
@@ -128,24 +129,26 @@ static void zfrelease(ZFMAP* zfmap, FD* fd);
 static void zfunlink(const char* canonpath);
 
 static int platformerr(int err);
-static int platformcreatefile(ZFMAP* map, const char* truepath,FD*);
-static int platformcreatedir(ZFMAP* map, const char* truepath);
-static int platformopenfile(ZFMAP* zfmap, const char* truepath, FD* fd);
-static int platformopendir(ZFMAP* map, const char* truepath);
-static int platformdircontent(ZFMAP* map, const char* path, NClist* contents);
-static int platformdelete(ZFMAP* map, const char* path, int delroot);
-static int platformseek(ZFMAP* map, FD* fd, int pos, size64_t* offset);
-static int platformread(ZFMAP* map, FD* fd, size64_t count, void* content);
-static int platformwrite(ZFMAP* map, FD* fd, size64_t count, const void* content);
-static void platformrelease(ZFMAP* zfmap, FD* fd);
-static int platformtestcontentbearing(ZFMAP* zfmap, const char* truepath);
+static int platformcreatefile(mode_t mode, const char* truepath,FD*);
+static int platformcreatedir(mode_t mode, const char* truepath);
+static int platformopenfile(mode_t mode, const char* truepath, FD* fd);
+static int platformopendir(mode_t mode, const char* truepath);
+static int platformdircontent(const char* path, NClist* contents);
+static int platformdelete(const char* path, int delroot);
+static int platformseek(FD* fd, int pos, size64_t* offset);
+static int platformread(FD* fd, size64_t count, void* content);
+static int platformwrite(FD* fd, size64_t count, const void* content);
+static void platformrelease(FD* fd);
+static int platformtestcontentbearing(const char* truepath);
 
 #ifdef VERIFY
 static int verify(const char* path, int isdir);
+static int verifykey(const char* key, int isdir);
 #endif
 
 static int zfinitialized = 0;
-static void zfinitialize(void)
+static void
+zfileinitialize(void)
 {
     if(!zfinitialized) {
         ZTRACE(5,NULL);
@@ -180,13 +183,14 @@ zfilecreate(const char *path, int mode, size64_t flags, void* parameters, NCZMAP
 {
     int stat = NC_NOERR;
     char* canonpath = NULL;
+    char* abspath = NULL;
     ZFMAP* zfmap = NULL;
     NCURI* url = NULL;
 	
     NC_UNUSED(parameters);
     ZTRACE(5,"path=%s mode=%d flag=%llu",path,mode,flags);
 
-    if(!zfinitialized) zfinitialize();
+    if(!zfinitialized) zfileinitialize();
 
     /* Fixup mode flags */
     mode |= (NC_NETCDF4 | NC_WRITE);
@@ -200,8 +204,13 @@ zfilecreate(const char *path, int mode, size64_t flags, void* parameters, NCZMAP
     if(strcasecmp(url->protocol,"file") != 0)
         {stat = NC_EURL; goto done;}
 
-    /* Canonicalize the root path */
-    if((stat = NCpathcanonical(url->path,&canonpath))) goto done;
+    /* Convert the root path */
+    if((canonpath = NCpathcvt(url->path))==NULL)
+	{stat = NC_ENOMEM; goto done;}
+
+    /* Make the root path be absolute */
+    if((abspath = NCpathabsolute(canonpath)) == NULL)
+	{stat = NC_EURL; goto done;}
 
     /* Build the zmap state */
     if((zfmap = calloc(1,sizeof(ZFMAP))) == NULL)
@@ -213,15 +222,15 @@ zfilecreate(const char *path, int mode, size64_t flags, void* parameters, NCZMAP
     /* create => NC_WRITE */
     zfmap->map.mode = mode;
     zfmap->map.api = &zapi;
-    zfmap->root = canonpath;
-        canonpath = NULL;
+    zfmap->root = abspath;
+        abspath = NULL;
 
     /* If NC_CLOBBER, then delete below file tree */
     if(!fIsSet(mode,NC_NOCLOBBER))
-	platformdelete(zfmap,zfmap->root,0);
+	platformdelete(zfmap->root,0);
     
     /* make sure we can access the root directory; create if necessary */
-    if((stat = platformcreatedir(zfmap, zfmap->root)))
+    if((stat = platformcreatedir(zfmap->map.mode, zfmap->root)))
 	goto done;
 
     /* Dataset superblock will be written by higher layer */
@@ -231,6 +240,7 @@ zfilecreate(const char *path, int mode, size64_t flags, void* parameters, NCZMAP
 done:
     ncurifree(url);
     nullfree(canonpath);
+    nullfree(abspath);
     if(stat)
     	zfileclose((NCZMAP*)zfmap,1);
     return ZUNTRACE(stat);
@@ -250,13 +260,14 @@ zfileopen(const char *path, int mode, size64_t flags, void* parameters, NCZMAP**
 {
     int stat = NC_NOERR;
     char* canonpath = NULL;
+    char* abspath = NULL;
     ZFMAP* zfmap = NULL;
     NCURI*url = NULL;
     
     NC_UNUSED(parameters);
     ZTRACE(5,"path=%s mode=%d flags=%llu",path,mode,flags);
 
-    if(!zfinitialized) zfinitialize();
+    if(!zfinitialized) zfileinitialize();
 
     /* Fixup mode flags */
     mode = (NC_NETCDF4 | mode);
@@ -267,8 +278,13 @@ zfileopen(const char *path, int mode, size64_t flags, void* parameters, NCZMAP**
     if(strcasecmp(url->protocol,"file") != 0)
         {stat = NC_EURL; goto done;}
 
-    /* Canonicalize the root path */
-    if((stat = NCpathcanonical(url->path,&canonpath))) goto done;
+    /* Convert the root path */
+    if((canonpath = NCpathcvt(url->path))==NULL)
+	{stat = NC_ENOMEM; goto done;}
+
+    /* Make the root path be absolute */
+    if((abspath = NCpathabsolute(canonpath)) == NULL)
+	{stat = NC_EURL; goto done;}
 
     /* Build the zmap state */
     if((zfmap = calloc(1,sizeof(ZFMAP))) == NULL)
@@ -279,11 +295,11 @@ zfileopen(const char *path, int mode, size64_t flags, void* parameters, NCZMAP**
     zfmap->map.flags = flags;
     zfmap->map.mode = mode;
     zfmap->map.api = (NCZMAP_API*)&zapi;
-    zfmap->root = canonpath;
-	canonpath = NULL;
+    zfmap->root = abspath;
+	abspath = NULL;
     
     /* Verify root dir exists */
-    if((stat = platformopendir(zfmap,zfmap->root)))
+    if((stat = platformopendir(zfmap->map.mode,zfmap->root)))
 	goto done;
     
     /* Dataset superblock will be read by higher layer */
@@ -293,10 +309,26 @@ zfileopen(const char *path, int mode, size64_t flags, void* parameters, NCZMAP**
 done:
     ncurifree(url);
     nullfree(canonpath);
+    nullfree(abspath);
     if(stat) zfileclose((NCZMAP*)zfmap,0);
     return ZUNTRACE(stat);
 }
 
+static int
+zfiletruncate(const char* surl)
+{
+    int stat = NC_NOERR;
+    NCURI* url = NULL;
+
+    ZTRACE(6,"url=%s",surl);
+    ncuriparse(surl,&url);
+    if(url == NULL) {stat = NC_EURL; goto done;}
+    platformdelete(url->path,0); /* leave root; ignore errors */
+done:
+    ncurifree(url);
+    return stat;    
+}
+
 /**************************************************/
 /* Object API */
 
@@ -331,7 +363,7 @@ zfilelen(NCZMAP* map, const char* key, size64_t* lenp)
     switch (stat=zflookupobj(zfmap,key,&fd)) {
     case NC_NOERR:
         /* Get file size */
-        if((stat=platformseek(zfmap, &fd, SEEK_END, &len))) goto done;
+        if((stat=platformseek(&fd, SEEK_END, &len))) goto done;
 	break;
     case NC_ENOOBJECT: stat = NC_EEMPTY;
     case NC_EEMPTY: break;
@@ -354,14 +386,14 @@ zfileread(NCZMAP* map, const char* key, size64_t start, size64_t count, void* co
     ZTRACE(5,"map=%s key=%s start=%llu count=%llu",map->url,key,start,count);
 
 #ifdef VERIFY
-    if(!verify(key,!FLAG_ISDIR))
+    if(!verifykey(key,!FLAG_ISDIR))
         assert(!"expected file, have dir");
 #endif
 
     switch (stat = zflookupobj(zfmap,key,&fd)) {
     case NC_NOERR:
-        if((stat = platformseek(zfmap, &fd, SEEK_SET, &start))) goto done;
-        if((stat = platformread(zfmap, &fd, count, content))) goto done;
+        if((stat = platformseek(&fd, SEEK_SET, &start))) goto done;
+        if((stat = platformread(&fd, count, content))) goto done;
 	break;
     case NC_ENOOBJECT: stat = NC_EEMPTY;
     case NC_EEMPTY: break;
@@ -374,17 +406,18 @@ done:
 }
 
 static int
-zfilewrite(NCZMAP* map, const char* key, size64_t start, size64_t count, const void* content)
+zfilewrite(NCZMAP* map, const char* key, size64_t count, const void* content)
 {
     int stat = NC_NOERR;
     FD fd = FDNUL;
     ZFMAP* zfmap = (ZFMAP*)map; /* cast to true type */
     char* truepath = NULL;
+    size64_t start = 0;
 
     ZTRACE(5,"map=%s key=%s start=%llu count=%llu",map->url,key,start,count);
 
 #ifdef VERIFY
-    if(!verify(key,!FLAG_ISDIR))
+    if(!verifykey(key,!FLAG_ISDIR))
         assert(!"expected file, have dir");
 #endif
 
@@ -397,11 +430,11 @@ zfilewrite(NCZMAP* map, const char* key, size64_t start, size64_t count, const v
         /* Create truepath */
         if((stat = zffullpath(zfmap,key,&truepath))) goto done;
 	/* Create file */
-	if((stat = platformcreatefile(zfmap,truepath,&fd))) goto done;
+	if((stat = platformcreatefile(zfmap->map.mode,truepath,&fd))) goto done;
 	/* Fall thru to write the object */
     case NC_NOERR:
-        if((stat = platformseek(zfmap, &fd, SEEK_SET, &start))) goto done;
-        if((stat = platformwrite(zfmap, &fd, count, content))) goto done;
+        if((stat = platformseek(&fd, SEEK_SET, &start))) goto done;
+        if((stat = platformwrite(&fd, count, content))) goto done;
 	break;
     default: break;
     }
@@ -423,7 +456,7 @@ zfileclose(NCZMAP* map, int delete)
     
     /* Delete the subtree below the root and the root */
     if(delete) {
-	stat = platformdelete(zfmap,zfmap->root,1);
+	stat = platformdelete(zfmap->root,1);
 	zfunlink(zfmap->root);
     }
     nczm_clear(map);
@@ -456,7 +489,7 @@ zfilesearch(NCZMAP* map, const char* prefixkey, NClist* matches)
     else if((stat = nczm_concat(zfmap->root,prefixkey,&fullpath))) goto done;
 
     /* get names of the next level path entries */
-    switch (stat = platformdircontent(zfmap, fullpath, nextlevel)) {
+    switch (stat = platformdircontent(fullpath, nextlevel)) {
     case NC_NOERR: /* ok */
 	break;
     case NC_EEMPTY: /* not a dir */
@@ -497,7 +530,8 @@ static int
 zfcreategroup(ZFMAP* zfmap, const char* key, int nskip)
 {
     int stat = NC_NOERR;
-    int i, len;
+    size_t i;
+    int len;
     char* fullpath = NULL;
     NCbytes* path = ncbytesnew();
     NClist* segments = nclistnew();
@@ -513,7 +547,7 @@ zfcreategroup(ZFMAP* zfmap, const char* key, int nskip)
 	ncbytescat(path,"/");
 	ncbytescat(path,seg);
 	/* open and optionally create the directory */	
-	stat = platformcreatedir(zfmap,ncbytescontents(path));
+	stat = platformcreatedir(zfmap->map.mode,ncbytescontents(path));
 	if(stat) goto done;
     }
 done:
@@ -540,11 +574,11 @@ zflookupobj(ZFMAP* zfmap, const char* key, FD* fd)
 	{goto done;}    
 
     /* See if this is content-bearing */
-    if((stat = platformtestcontentbearing(zfmap,path)))
+    if((stat = platformtestcontentbearing(path)))
 	goto done;        
 
     /* Open the file */
-    if((stat = platformopenfile(zfmap,path,fd)))
+    if((stat = platformopenfile(zfmap->map.mode,path,fd)))
         goto done;
 
 done:
@@ -558,7 +592,7 @@ static void
 zfrelease(ZFMAP* zfmap, FD* fd)
 {
     ZTRACE(5,"map=%s fd=%d",zfmap->map.url,(fd?fd->fd:-1));
-    platformrelease(zfmap,fd);
+    platformrelease(fd);
     (void)ZUNTRACE(NC_NOERR);
 }
 
@@ -570,6 +604,7 @@ NCZMAP_DS_API zmap_file = {
     0,
     zfilecreate,
     zfileopen,
+    zfiletruncate,
 };
 
 static NCZMAP_API zapi = {
@@ -647,7 +682,7 @@ platformerr(int err)
 @return NC_ENOOBJECT if not found
 */
 static int
-platformtestcontentbearing(ZFMAP* zfmap, const char* canonpath)
+platformtestcontentbearing(const char* canonpath)
 {
     int ret = 0;
     struct stat buf;
@@ -669,12 +704,11 @@ platformtestcontentbearing(ZFMAP* zfmap, const char* canonpath)
 
 /* Create a file */
 static int
-platformcreatefile(ZFMAP* zfmap, const char* canonpath, FD* fd)
+platformcreatefile(mode_t mode, const char* canonpath, FD* fd)
 {
     int stat = NC_NOERR;
     int ioflags = 0;
     int createflags = 0;
-    int mode = zfmap->map.mode;
     int permissions = NC_DEFAULT_ROPEN_PERMS;
 
     ZTRACE(6,"map=%s canonpath=%s",zfmap->map.url,canonpath);
@@ -711,11 +745,10 @@ done:
 
 /* Open a file; fail if it does not exist */
 static int
-platformopenfile(ZFMAP* zfmap, const char* canonpath, FD* fd)
+platformopenfile(mode_t mode, const char* canonpath, FD* fd)
 {
     int stat = NC_NOERR;
     int ioflags = 0;
-    int mode = zfmap->map.mode;
     int permissions = 0;
 
     ZTRACE(6,"map=%s canonpath=%s",zfmap->map.url,canonpath);
@@ -740,9 +773,7 @@ platformopenfile(ZFMAP* zfmap, const char* canonpath, FD* fd)
     /* Try to open file  (will localize) */
     fd->fd = NCopen3(canonpath, ioflags, permissions);
     if(fd->fd < 0)
-        {
-fprintf(stderr,"xxx: canonpath=%s\n",canonpath);
-stat = platformerr(errno); goto done;} /* could not open */
+        {stat = platformerr(errno); goto done;} /* could not open */
 done:
     errno = 0;
     return ZUNTRACEX(stat,"fd=%d",(fd?fd->fd:-1));
@@ -750,10 +781,9 @@ done:
 
 /* Create a dir */
 static int
-platformcreatedir(ZFMAP* zfmap, const char* canonpath)
+platformcreatedir(mode_t mode, const char* canonpath)
 {
     int ret = NC_NOERR;
-    int mode = zfmap->map.mode;
 
     ZTRACE(6,"map=%s canonpath=%s",zfmap->map.url,canonpath);
 
@@ -781,7 +811,7 @@ done:
 
 /* Open a dir; fail if it does not exist */
 static int
-platformopendir(ZFMAP* zfmap, const char* canonpath)
+platformopendir(mode_t mode, const char* canonpath)
 {
     int ret = NC_NOERR;
 
@@ -812,7 +842,7 @@ There are several possibilities:
 
 #ifdef _WIN32
 static int
-platformdircontent(ZFMAP* zfmap, const char* canonpath, NClist* contents)
+platformdircontent(const char* canonpath, NClist* contents)
 {
     int ret = NC_NOERR;
     errno = 0;
@@ -825,7 +855,7 @@ platformdircontent(ZFMAP* zfmap, const char* canonpath, NClist* contents)
 
     ZTRACE(6,"map=%s canonpath=%s",zfmap->map.url,canonpath);
 
-    switch (ret = platformtestcontentbearing(zfmap, canonpath)) {
+    switch (ret = platformtestcontentbearing(canonpath)) {
     case NC_EEMPTY: ret = NC_NOERR; break; /* directory */    
     case NC_NOERR: ret = NC_EEMPTY; goto done;
     default: goto done;
@@ -860,7 +890,6 @@ platformdircontent(ZFMAP* zfmap, const char* canonpath, NClist* contents)
 	}
     }
     do {
-	char* p = NULL;
 	const char* name = NULL;
         name = FindFileData.cFileName;
 	if(strcmp(name,".")==0 || strcmp(name,"..")==0)
@@ -880,7 +909,7 @@ done:
 #else /*!_WIN32*/
 
 static int
-platformdircontent(ZFMAP* zfmap, const char* canonpath, NClist* contents)
+platformdircontent(const char* canonpath, NClist* contents)
 {
     int ret = NC_NOERR;
     errno = 0;
@@ -888,9 +917,10 @@ platformdircontent(ZFMAP* zfmap, const char* canonpath, NClist* contents)
 
     ZTRACE(6,"map=%s canonpath=%s",zfmap->map.url,canonpath);
 
-    switch (ret = platformtestcontentbearing(zfmap, canonpath)) {
+    switch (ret = platformtestcontentbearing(canonpath)) {
     case NC_EEMPTY: ret = NC_NOERR; break; /* directory */    
     case NC_NOERR: ret = NC_EEMPTY; goto done; 
+    case NC_ENOOBJECT: ret = NC_EEMPTY; goto done;
     default: goto done;
     }
 
@@ -916,73 +946,11 @@ done:
 }
 #endif /*_WIN32*/
 
-#if 0
 static int
-platformdeleter(ZFMAP* zfmap, NClist* segments, int depth)
+platformdeleter(NCbytes* canonpath, int depth)
 {
     int ret = NC_NOERR;
-    struct stat statbuf;
-    struct dirent* entry = NULL;
-    DIR* dir = NULL;
-    char* path = NULL;
-    char* tmp = NULL;
-
-    if((ret = nczm_join(segments,&path))) goto done;
-
-    /* When running on any platform that can accept drive letters */
-    if((ret = nczm_fixpath(path,&tmp))) goto done;
-    nullfree(path); path = NULL;
-    if((path = NCpathcvt(tmp))==NULL) {ret = NC_ENOMEM; goto done;}
-
-    errno = 0;
-    ret = NCstat(path, &statbuf);
-    if(ret < 0) {
-        if(errno == ENOENT) {ret = NC_NOERR; goto done;}
-	else {ret = platformerr(errno); goto done;}
-    }
-    /* process this file */
-    if(S_ISDIR(statbuf.st_mode)) {
-        if((dir = NCopendir(path)) == NULL)
-	     {ret = platformerr(errno); goto done;}
-        for(;;) {
-	    char* seg = NULL;
-	    errno = 0;
-            entry = readdir(dir);
-	    if(entry == NULL) {ret = platformerr(errno); break;}
-	    /* Ignore "." and ".." */
-	    if(strcmp(entry->d_name,".")==0) continue;
-    	    if(strcmp(entry->d_name,"..")==0) continue;
-	    /* append name to segments */
-	    if((seg = strdup(entry->d_name)) == NULL)
-		{ret = NC_ENOMEM; goto done;}
-	    nclistpush(segments,seg);
-	    /* recurse */
-	    if((ret = platformdeleter(zfmap, segments, depth+1))) goto done;
-	    /* remove+reclaim last segment */
-	    nclistpop(segments);
-	    nullfree(seg);	    	    
-        }
-	/* Delete this file */
-	rmdir(path);
-    } else {
-	assert(S_ISREG(statbuf.st_mode));
-	unlink(path);
-    }
-done:
-    if(dir) NCclosedir(dir);
-    /* delete this file|dir */
-    ret = NCremove(path);
-    nullfree(path);
-    nullfree(tmp);
-    return ZUNTRACE(ret);
-}
-#endif /*0*/
-
-static int
-platformdeleter(ZFMAP* zfmap, NCbytes* canonpath, int delroot, int depth)
-{
-    int ret = NC_NOERR;
-    int i;
+    size_t i;
     NClist* subfiles = nclistnew();
     size_t tpathlen = ncbyteslength(canonpath);
     char* local = NULL;
@@ -990,7 +958,7 @@ platformdeleter(ZFMAP* zfmap, NCbytes* canonpath, int delroot, int depth)
     local = ncbytescontents(canonpath);
     ZTRACE(6,"map=%s canonpath=%s delroot=%d depth=%d",zfmap->map.url,local,delroot,depth);
 
-    ret = platformdircontent(zfmap, local, subfiles);
+    ret = platformdircontent(local, subfiles);
 #ifdef DEBUG
     {int i;
 	fprintf(stderr,"xxx: contents:\n");
@@ -1007,12 +975,12 @@ platformdeleter(ZFMAP* zfmap, NCbytes* canonpath, int delroot, int depth)
             ncbytescat(canonpath, "/");
             ncbytescat(canonpath, name);
             /* recurse */
-            if ((ret = platformdeleter(zfmap, canonpath,delroot,depth+1))) goto done;
+            if ((ret = platformdeleter(canonpath,depth+1))) goto done;
             ncbytessetlength(canonpath,tpathlen); /* reset */
 	    ncbytesnull(canonpath);
 	    local = ncbytescontents(canonpath);
 	}
-	if(depth > 0 || delroot) {
+	if(depth > 0) {
 #ifdef DEBUG
 fprintf(stderr,"xxx: remove:  %s\n",canonpath);
 #endif
@@ -1030,13 +998,13 @@ fprintf(stderr,"xxx: remove: errno=%d|%s\n",errno,nc_strerror(errno));
 #ifdef DEBUG
 fprintf(stderr,"xxx: remove:  %s\n",canonpath);
 #endif
-        if(NCremove(local) < 0) {/* kill this file */
+            if(NCremove(local) < 0) {/* kill this file */
 #ifdef DEBUG
 fprintf(stderr,"xxx: remove: errno=%d|%s\n",errno,nc_strerror(errno));
 #endif
-	    ret = errno;
-	    goto done;
-	}
+	        ret = errno;
+	        goto done;
+	    }
 	break;
     case NC_ENOTFOUND:
     default:
@@ -1053,7 +1021,7 @@ done:
 
 /* Deep file/dir deletion; depth first */
 static int
-platformdelete(ZFMAP* zfmap, const char* rootpath, int delroot)
+platformdelete(const char* rootpath, int delroot)
 {
     int stat = NC_NOERR;
     NCbytes* canonpath = ncbytesnew();
@@ -1064,7 +1032,19 @@ platformdelete(ZFMAP* zfmap, const char* rootpath, int delroot)
     ncbytescat(canonpath,rootpath);
     if(rootpath[strlen(rootpath)-1] == '/') /* elide trailing '/' */
 	ncbytessetlength(canonpath,ncbyteslength(canonpath)-1);
-    if((stat = platformdeleter(zfmap,canonpath,delroot,0))) goto done;
+    /* See if file even exists */
+    stat = NCaccess(ncbytescontents(canonpath),F_OK);
+    if(stat < 0) {
+	stat = errno;
+	goto done;
+    }
+    if((stat = platformdeleter(canonpath,0))) goto done;
+    if(delroot) {
+        if(NCrmdir(rootpath) < 0) { /* kill this dir */
+    	    stat = errno;
+	    goto done;
+	 }
+    }
 done:
     ncbytesfree(canonpath);
     errno = 0;
@@ -1072,7 +1052,7 @@ done:
 }
 
 static int
-platformseek(ZFMAP* zfmap, FD* fd, int pos, size64_t* sizep)
+platformseek(FD* fd, int pos, size64_t* sizep)
 {
     int ret = NC_NOERR;
     off_t size, newsize;
@@ -1095,7 +1075,7 @@ done:
 }
 
 static int
-platformread(ZFMAP* zfmap, FD* fd, size64_t count, void* content)
+platformread(FD* fd, size64_t count, void* content)
 {
     int stat = NC_NOERR;
     size_t need = count;
@@ -1118,7 +1098,7 @@ done:
 }
 
 static int
-platformwrite(ZFMAP* zfmap, FD* fd, size64_t count, const void* content)
+platformwrite(FD* fd, size64_t count, const void* content)
 {
     int ret = NC_NOERR;
     size_t need = count;
@@ -1156,7 +1136,7 @@ platformcwd(char** cwdp)
    equivalent to closing the file descriptor.
 */
 static void
-platformrelease(ZFMAP* zfmap, FD* fd)
+platformrelease(FD* fd)
 {
     ZTRACE(6,"map=%s fd=%d",zfmap->map.url,(fd?fd->fd:-1));
     if(fd->fd >=0) NCclose(fd->fd);
@@ -1168,7 +1148,7 @@ platformrelease(ZFMAP* zfmap, FD* fd)
 /* Close FD => return typ to FDNONE */
 */
 static void
-platformclose(ZFMAP* zfmap, FD* fd)
+platformclose(FD* fd)
 {
     if(fd->typ == FDFILE) {
         if(fd->fd >=0) close(fd->u,fd);
@@ -1197,6 +1177,24 @@ verify(const char* path, int isdir)
     if(!isdir && S_ISREG(buf.st_mode)) return 1;           
     return 0;
 }
+
+static int
+verifykey(const char* key, int isdir)
+{
+    int ret = 0;
+    struct stat buf;
+
+    if(key[0] == '/') key++; /* Want relative name */
+
+    ret = NCaccess(key,ACCESS_MODE_EXISTS);
+    if(ret < 0)
+        return 1; /* If it does not exist, then it can be anything */
+    ret = NCstat(key,&buf);
+    if(ret < 0) abort();
+    if(isdir && S_ISDIR(buf.st_mode)) return 1;
+    if(!isdir && S_ISREG(buf.st_mode)) return 1;           
+    return 0;
+}
 #endif
 
 #if 0
