@@ -14,6 +14,7 @@
 #include "zcache.h"
 #include "ncxcache.h"
 #include "zfilter.h"
+#include <stddef.h>
 
 #undef DEBUG
 
@@ -21,12 +22,20 @@
 
 #define LEAFLEN 32
 
+#define USEPARAMSIZE 0xffffffffffffffff
+
 /* Forward */
 static int get_chunk(NCZChunkCache* cache, NCZCacheEntry* entry);
 static int put_chunk(NCZChunkCache* cache, NCZCacheEntry*);
-static int makeroom(NCZChunkCache* cache);
+static int verifycache(NCZChunkCache* cache);
 static int flushcache(NCZChunkCache* cache);
-static int constraincache(NCZChunkCache* cache);
+static int constraincache(NCZChunkCache* cache, size64_t needed);
+
+static void
+setmodified(NCZCacheEntry* e, int tf)
+{
+    e->modified = tf;
+}
 
 /**************************************************/
 /* Dispatch table per-var cache functions */
@@ -69,7 +78,7 @@ NCZ_set_var_chunk_cache(int ncid, int varid, size_t cachesize, size_t nelems, fl
     assert(grp && h5);
 
     /* Find the var. */
-    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, varid)))
+    if (!(var = (NC_VAR_INFO_T *)ncindexith(grp->vars, (size_t)varid)))
         {retval = NC_ENOTVAR; goto done;}
     assert(var && var->hdr.id == varid);
 
@@ -120,8 +129,9 @@ fprintf(stderr,"xxx: adjusting cache for: %s\n",var->hdr.name);
     /* Reclaim any existing fill_chunk */
     if((stat = NCZ_reclaim_fill_chunk(zcache))) goto done;
     /* Reset the parameters */
-    zvar->cache->maxsize = var->chunkcache.size;
-    zvar->cache->maxentries = var->chunkcache.nelems;
+    zvar->cache->params.size = var->chunkcache.size;
+    zvar->cache->params.nelems = var->chunkcache.nelems;
+    zvar->cache->params.preemption = var->chunkcache.preemption;
 #ifdef DEBUG
     fprintf(stderr,"%s.cache.adjust: size=%ld nelems=%ld\n",
         var->hdr.name,(unsigned long)zvar->cache->maxsize,(unsigned long)zvar->cache->maxentries);
@@ -130,7 +140,7 @@ fprintf(stderr,"xxx: adjusting cache for: %s\n",var->hdr.name);
     zcache->chunksize = zvar->chunksize;
     zcache->chunkcount = 1;
     if(var->ndims > 0) {
-	int i;
+	size_t i;
 	for(i=0;i<var->ndims;i++) {
 	    zcache->chunkcount *= var->chunksizes[i];
         }
@@ -163,7 +173,6 @@ NCZ_create_chunk_cache(NC_VAR_INFO_T* var, size64_t chunksize, char dimsep, NCZC
     if(chunksize == 0) return NC_EINVAL;
 
     zvar = (NCZ_VAR_INFO_T*)var->format_var_info;
-    
     if((cache = calloc(1,sizeof(NCZChunkCache))) == NULL)
 	{stat = NC_ENOMEM; goto done;}
     cache->var = var;
@@ -175,12 +184,15 @@ NCZ_create_chunk_cache(NC_VAR_INFO_T* var, size64_t chunksize, char dimsep, NCZC
 
     cache->chunkcount = 1;
     if(var->ndims > 0) {
-	int i;
+	size_t i;
 	for(i=0;i<var->ndims;i++) {
 	    cache->chunkcount *= var->chunksizes[i];
         }
     }
     
+    /* Set default cache parameters */
+    cache->params = NC_getglobalstate()->chunkcache;
+
 #ifdef FLUSH
     cache->maxentries = 1;
 #endif
@@ -192,7 +204,7 @@ NCZ_create_chunk_cache(NC_VAR_INFO_T* var, size64_t chunksize, char dimsep, NCZC
     if((stat = ncxcachenew(LEAFLEN,&cache->xcache))) goto done;
     if((cache->mru = nclistnew()) == NULL)
 	{stat = NC_ENOMEM; goto done;}
-    nclistsetalloc(cache->mru,cache->maxentries);
+    nclistsetalloc(cache->mru,cache->params.nelems);
 
     if(cachep) {*cachep = cache; cache = NULL;}
 done:
@@ -202,9 +214,13 @@ done:
 }
 
 static void
-free_cache_entry(NCZCacheEntry* entry)
+free_cache_entry(NCZChunkCache* cache, NCZCacheEntry* entry)
 {
     if(entry) {
+        int tid = cache->var->type_info->hdr.id;
+	if(tid == NC_STRING && !entry->isfixedstring) {
+            NC_reclaim_data(cache->var->container->nc4_info->controller,tid,entry->data,cache->chunkcount);
+	}
 	nullfree(entry->data);
 	nullfree(entry->key.varkey);
 	nullfree(entry->key.chunkkey);
@@ -225,7 +241,7 @@ NCZ_free_chunk_cache(NCZChunkCache* cache)
         NCZCacheEntry* entry = nclistremove(cache->mru,0);
 	(void)ncxcacheremove(cache->xcache,entry->hashkey,&ptr);
 	assert(ptr == entry);
-	free_cache_entry(entry);
+        free_cache_entry(cache,entry);
     }
 #ifdef DEBUG
 fprintf(stderr,"|cache.free|=%ld\n",nclistlength(cache->mru));
@@ -281,7 +297,7 @@ NCZ_read_cache_chunk(NCZChunkCache* cache, const size64_t* indices, void** datap
 	/* Create a new entry */
 	if((entry = calloc(1,sizeof(NCZCacheEntry)))==NULL)
 	    {stat = NC_ENOMEM; goto done;}
-	memcpy(entry->indices,indices,rank*sizeof(size64_t));
+	memcpy(entry->indices,indices,(size_t)rank*sizeof(size64_t));
         /* Create the key for this cache */
         if((stat = NCZ_buildchunkpath(cache,indices,&entry->key))) goto done;
         entry->hashkey = hkey;
@@ -290,7 +306,7 @@ NCZ_read_cache_chunk(NCZChunkCache* cache, const size64_t* indices, void** datap
 	if((stat=get_chunk(cache,entry))) goto done;
 	assert(entry->data != NULL);
 	/* Ensure cache constraints not violated; but do it before entry is added */
-	if((stat=makeroom(cache))) goto done;
+	if((stat=verifycache(cache))) goto done;
         nclistpush(cache->mru,entry);
 	if((stat = ncxcacheinsert(cache->xcache,entry->hashkey,entry))) goto done;
     }
@@ -303,7 +319,7 @@ fprintf(stderr,"|cache.read.lru|=%ld\n",nclistlength(cache->mru));
     
 done:
     if(created && stat == NC_NOERR)  stat = NC_EEMPTY; /* tell upper layers */
-    if(entry) free_cache_entry(entry);
+    if(entry) free_cache_entry(cache,entry);
     return THROW(stat);
 }
 
@@ -332,7 +348,7 @@ NCZ_write_cache_chunk(NCZChunkCache* cache, const size64_t* indices, void* conte
 	    {stat = NC_ENOMEM; goto done;}
 	memcpy(entry->data,content,cache->chunksize);
     }
-    entry->modified = 1;
+    setmodified(entry,1);
     nclistpush(cache->mru,entry); /* MRU order */
 #ifdef DEBUG
 fprintf(stderr,"|cache.write|=%ld\n",nclistlength(cache->mru));
@@ -340,24 +356,26 @@ fprintf(stderr,"|cache.write|=%ld\n",nclistlength(cache->mru));
     entry = NULL;
 
     /* Ensure cache constraints not violated */
-    if((stat=makeroom(cache))) goto done;
+    if((stat=verifycache(cache))) goto done;
 
 done:
-    if(entry) free_cache_entry(entry);
+    if(entry) free_cache_entry(cache,entry);
     return THROW(stat);
 }
 #endif
 
-/* Constrain cache, but allow at least one entry */
+/* Constrain cache */
 static int
-makeroom(NCZChunkCache* cache)
+verifycache(NCZChunkCache* cache)
 {
     int stat = NC_NOERR;
 
+#if 0
     /* Sanity check; make sure at least one entry is always allowed */
     if(nclistlength(cache->mru) == 1)
 	goto done;
-    stat = constraincache(cache);
+#endif
+    if((stat = constraincache(cache,USEPARAMSIZE))) goto done;
 done:
     return stat;
 }
@@ -367,8 +385,16 @@ done:
 static int
 flushcache(NCZChunkCache* cache)
 {
-    cache->maxentries = 0;
-    return constraincache(cache);
+    int stat = NC_NOERR;
+#if 0
+    size_t oldsize = cache->params.size;
+    cache->params.size = 0;
+    stat = constraincache(cache,USEPARAMSIZE);
+    cache->params.size = oldsize;
+#else
+    stat = constraincache(cache,USEPARAMSIZE);
+#endif
+    return stat;
 }
 
 
@@ -376,21 +402,32 @@ flushcache(NCZChunkCache* cache)
    violating any of its constraints.
    On entry, constraints might be violated.
    Make sure that the entryinuse (NULL => no constraint) is not reclaimed.
+@param cache
+@param needed make sure there is room for this much space; USEPARAMSIZE => ensure no more than  cache params is used.
 */
 
 static int
-constraincache(NCZChunkCache* cache)
+constraincache(NCZChunkCache* cache, size64_t needed)
 {
     int stat = NC_NOERR;
+    size64_t final_size;
 
     /* If the cache is empty then do nothing */
     if(cache->used == 0) goto done;
 
+    if(needed == USEPARAMSIZE)
+        final_size = cache->params.size;
+    else if(cache->used > needed)
+        final_size = cache->used - needed;
+    else
+        final_size = 0;
+
     /* Flush from LRU end if we are at capacity */
-    while(nclistlength(cache->mru) > cache->maxentries || cache->used > cache->maxsize) {
-	int i;
+    while(nclistlength(cache->mru) > cache->params.nelems || cache->used > final_size) {
+	size_t i;
 	void* ptr;
 	NCZCacheEntry* e = ncxcachelast(cache->xcache); /* last entry is the least recently used */
+	if(e == NULL) break;
         if((stat = ncxcacheremove(cache->xcache,e->hashkey,&ptr))) goto done;
    	assert(e == ptr);
         for(i=0;i<nclistlength(cache->mru);i++) {
@@ -415,6 +452,12 @@ done:
     return stat;
 }
 
+/**
+Push modified cache entries to disk.
+Also make sure the cache size is correct.
+@param cache
+@return NC_EXXX error
+*/
 int
 NCZ_flush_chunk_cache(NCZChunkCache* cache)
 {
@@ -429,15 +472,21 @@ NCZ_flush_chunk_cache(NCZChunkCache* cache)
     for(i=0;i<nclistlength(cache->mru);i++) {
         NCZCacheEntry* entry = nclistget(cache->mru,i);
         if(entry->modified) {
-	    /* Make cache used be consistent across filter application */
-	    cache->used -= entry->size;
 	    /* Write out this chunk in toto*/
   	    if((stat=put_chunk(cache,entry)))
 	        goto done;
-	    cache->used += entry->size;
 	}
-        entry->modified = 0;
+        setmodified(entry,0);
     }
+    /* Re-compute space used */
+    cache->used = 0;
+    for(i=0;i<nclistlength(cache->mru);i++) {
+        NCZCacheEntry* entry = nclistget(cache->mru,i);
+        cache->used += entry->size;
+    }
+    /* Make sure cache size and nelems are correct */
+    if((stat=verifycache(cache))) goto done;
+
 
 done:
     return ZUNTRACE(stat);
@@ -447,8 +496,10 @@ done:
 int
 NCZ_ensure_fill_chunk(NCZChunkCache* cache)
 {
-    int i, stat = NC_NOERR;
+    int stat = NC_NOERR;
+    size_t i;
     NC_VAR_INFO_T* var = cache->var;
+    nc_type typeid = var->type_info->hdr.id;
     size_t typesize = var->type_info->size;
 
     if(cache->fillchunk) goto done;
@@ -461,6 +512,11 @@ NCZ_ensure_fill_chunk(NCZChunkCache* cache)
 	goto done;
     }
     if((stat = NCZ_ensure_fill_value(var))) goto done;
+    if(typeid == NC_STRING) {
+        char* src = *((char**)(var->fill_value));
+	char** dst = (char**)(cache->fillchunk);
+        for(i=0;i<cache->chunkcount;i++) dst[i] = strdup(src);
+    } else
     switch (typesize) {
     case 1: {
         unsigned char c = *((unsigned char*)var->fill_value);
@@ -497,37 +553,31 @@ NCZ_reclaim_fill_chunk(NCZChunkCache* zcache)
     int stat = NC_NOERR;
     if(zcache && zcache->fillchunk) {
 	NC_VAR_INFO_T* var = zcache->var;
-	int ncid = var->container->nc4_info->controller->ext_ncid;
 	int tid = var->type_info->hdr.id;
 	size_t chunkcount = zcache->chunkcount;
-        stat = nc_reclaim_data_all(ncid,tid,zcache->fillchunk,chunkcount);
+        stat = NC_reclaim_data_all(var->container->nc4_info->controller,tid,zcache->fillchunk,chunkcount);
 	zcache->fillchunk = NULL;
     }
     return stat;
 }
 
-#if 0
 int
-NCZ_chunk_cache_modified(NCZChunkCache* cache, const size64_t* indices)
+NCZ_chunk_cache_modify(NCZChunkCache* cache, const size64_t* indices)
 {
     int stat = NC_NOERR;
-    char* key = NULL;
+    ncexhashkey_t hkey = 0;
     NCZCacheEntry* entry = NULL;
-    int rank = cache->ndims;
 
-    /* Create the key for this cache */
-    if((stat=NCZ_buildchunkkey(rank, indices, &key))) goto done;
+    /* the hash key */
+    hkey = ncxcachekey(indices,sizeof(size64_t)*cache->ndims);
 
     /* See if already in cache */
-    if(NC_hashmapget(cache->mru, key, strlen(key), (uintptr_t*)entry)) { /* found */
-	entry->modified = 1;
-    }
+    if((stat=ncxcachelookup(cache->xcache, hkey, (void**)&entry))) {stat = NC_EINTERNAL; goto done;}
+    setmodified(entry,1);
 
 done:
-    nullfree(key);
     return THROW(stat);
 }
-#endif
 
 /**************************************************/
 /*
@@ -556,7 +606,7 @@ int
 NCZ_buildchunkkey(size_t R, const size64_t* chunkindices, char dimsep, char** keyp)
 {
     int stat = NC_NOERR;
-    int r;
+    size_t r;
     NCbytes* key = ncbytesnew();
 
     if(keyp) *keyp = NULL;
@@ -597,6 +647,8 @@ put_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     NCZ_FILE_INFO_T* zfile = NULL;
     NCZMAP* map = NULL;
     char* path = NULL;
+    nc_type tid = NC_NAT;
+    void* strchunk = NULL;
 
     ZTRACE(5,"cache.var=%s entry.key=%s",cache->var->hdr.name,entry->key);
     LOG((3, "%s: var: %p", __func__, cache->var));
@@ -605,7 +657,26 @@ put_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     zfile = file->format_file_info;
     map = zfile->map;
 
-#ifdef ENABLE_NCZARR_FILTERS
+    /* Collect some info */
+    tid = cache->var->type_info->hdr.id;
+
+    if(tid == NC_STRING && !entry->isfixedstring) {
+        /* Convert from char* to char[strlen] format */
+        int maxstrlen = NCZ_get_maxstrlen((NC_OBJ*)cache->var);
+        assert(maxstrlen > 0);
+        if((strchunk = malloc((size_t)cache->chunkcount * (size_t)maxstrlen))==NULL) {stat = NC_ENOMEM; goto done;}
+        /* copy char* to char[] format */
+        if((stat = NCZ_char2fixed((const char**)entry->data,strchunk,cache->chunkcount,maxstrlen))) goto done;
+        /* Reclaim the old chunk */
+        if((stat = NC_reclaim_data_all(file->controller,tid,entry->data,cache->chunkcount))) goto done;
+        entry->data = NULL;
+        entry->data = strchunk; strchunk = NULL;
+        entry->size = (cache->chunkcount * (size64_t)maxstrlen);
+        entry->isfixedstring = 1;
+    }
+
+
+#ifdef NETCDF_ENABLE_NCZARR_FILTERS
     /* Make sure the entry is in filtered state */
     if(!entry->isfiltered) {
         NC_VAR_INFO_T* var = cache->var;
@@ -626,7 +697,7 @@ put_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
 #endif
 
     path = NCZ_chunkpath(entry->key);
-    stat = nczmap_write(map,path,0,entry->size,entry->data);
+    stat = nczmap_write(map,path,entry->size,entry->data);
     nullfree(path); path = NULL;
 
     switch(stat) {
@@ -636,6 +707,7 @@ put_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     default: goto done;
     }
 done:
+    nullfree(strchunk);
     nullfree(path);
     return ZUNTRACE(stat);
 }
@@ -657,9 +729,12 @@ get_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     NCZMAP* map = NULL;
     NC_FILE_INFO_T* file = NULL;
     NCZ_FILE_INFO_T* zfile = NULL;
+    NC_TYPE_INFO_T* xtype = NULL;
+    char** strchunk = NULL;
     size64_t size;
     int empty = 0;
     char* path = NULL;
+    int tid;
 
     ZTRACE(5,"cache.var=%s entry.key=%s sep=%d",cache->var->hdr.name,entry->key,cache->dimension_separator);
     
@@ -670,22 +745,27 @@ get_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     map = zfile->map;
     assert(map);
 
+    /* Collect some info */
+    xtype = cache->var->type_info;
+    tid = xtype->hdr.id;
+
     /* get size of the "raw" data on "disk" */
     path = NCZ_chunkpath(entry->key);
     stat = nczmap_len(map,path,&size);
     nullfree(path); path = NULL;
     switch(stat) {
-    case NC_NOERR: break;
+    case NC_NOERR: entry->size = size; break;
     case NC_EEMPTY: empty = 1; stat = NC_NOERR; break;
     default: goto done;
     }
 
+    /* make room in the cache */
+    if((stat = constraincache(cache,size))) goto done;    
+
     if(!empty) {
         /* Make sure we have a place to read it */
-        entry->size = size;
-        entry->isfiltered = FILTERED(cache); /* Is the data being read filtered? */
-        if((entry->data = (void*)malloc(entry->size)) == NULL)
-            {stat = NC_ENOMEM; goto done;}
+        if((entry->data = (void*)calloc(1,entry->size)) == NULL)
+	    {stat = NC_ENOMEM; goto done;}
 	/* Read the raw data */
         path = NCZ_chunkpath(entry->key);
         stat = nczmap_read(map,path,0,entry->size,(char*)entry->data);
@@ -695,27 +775,32 @@ get_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
         case NC_EEMPTY: empty = 1; stat = NC_NOERR;break;
 	default: goto done;
 	}
+        entry->isfiltered = (int)FILTERED(cache); /* Is the data being read filtered? */
+	if(tid == NC_STRING)
+	    entry->isfixedstring = 1; /* fill cache is in char[maxstrlen] format */
     }
     if(empty) {
 	/* fake the chunk */
-        entry->modified = (file->no_write?0:1);
+        setmodified(entry,(file->no_write?0:1));
 	entry->size = cache->chunksize;
-        if((entry->data = (void*)malloc(entry->size)) == NULL)
-            {stat = NC_ENOMEM; goto done;}
+	entry->data = NULL;
+        entry->isfixedstring = 0;
+        entry->isfiltered = 0;
         /* apply fill value */
 	if(cache->fillchunk == NULL)
 	    {if((stat = NCZ_ensure_fill_chunk(cache))) goto done;}
-	memcpy(entry->data,cache->fillchunk,entry->size);
-        entry->isfiltered = 0;
+	if((entry->data = calloc(1,entry->size))==NULL) {stat = NC_ENOMEM; goto done;}
+	if((stat = NCZ_copy_data(file,cache->var,cache->fillchunk,cache->chunkcount,ZREADING,entry->data))) goto done;
 	stat = NC_NOERR;
     }
-#ifdef ENABLE_NCZARR_FILTERS
+#ifdef NETCDF_ENABLE_NCZARR_FILTERS
     /* Make sure the entry is in unfiltered state */
-    if(entry->isfiltered) {
+    if(!empty && entry->isfiltered) {
         NC_VAR_INFO_T* var = cache->var;
         void* unfiltered = NULL; /* pointer to the unfiltered data */
         void* filtered = NULL; /* pointer to the filtered data */
 	size_t unflen; /* length of unfiltered data */
+	assert(tid != NC_STRING || entry->isfixedstring);
 	/* Get the filter chain to apply */
 	NClist* filterchain = (NClist*)var->filters;
 	if(nclistlength(filterchain) == 0) {stat = NC_EFILTER; goto done;}
@@ -730,7 +815,27 @@ get_chunk(NCZChunkCache* cache, NCZCacheEntry* entry)
     }
 #endif
 
+    if(tid == NC_STRING && entry->isfixedstring) {
+        /* Convert from char[strlen] to char* format */
+	int maxstrlen = NCZ_get_maxstrlen((NC_OBJ*)cache->var);
+	assert(maxstrlen > 0);
+	/* copy char[] to char* format */
+	if((strchunk = (char**)malloc(sizeof(char*)*cache->chunkcount))==NULL)
+  	    {stat = NC_ENOMEM; goto done;}
+	if((stat = NCZ_fixed2char(entry->data,strchunk,cache->chunkcount,maxstrlen))) goto done;
+	/* Reclaim the old chunk */
+	nullfree(entry->data);
+	entry->data = NULL;
+	entry->data = strchunk; strchunk = NULL;
+	entry->size = cache->chunkcount * sizeof(char*);
+	entry->isfixedstring = 0;
+    }
+
+    /* track new chunk */
+    cache->used += entry->size;
+
 done:
+    nullfree(strchunk);
     nullfree(path);
     return ZUNTRACE(stat);
 }
@@ -761,7 +866,7 @@ NCZ_dumpxcacheentry(NCZChunkCache* cache, NCZCacheEntry* e, NCbytes* buf)
 {
     char s[8192];
     char idx[64];
-    int i;
+    size_t i;
 
     ncbytescat(buf,"{");
     snprintf(s,sizeof(s),"modified=%u isfiltered=%u indices=",
@@ -787,7 +892,7 @@ NCZ_printxcache(NCZChunkCache* cache)
     static char xs[20000];
     NCbytes* buf = ncbytesnew();
     char s[8192];
-    int i;
+    size_t i;
 
     ncbytescat(buf,"NCZChunkCache:\n");
     snprintf(s,sizeof(s),"\tvar=%s\n\tndims=%u\n\tchunksize=%u\n\tchunkcount=%u\n\tfillchunk=%p\n",
@@ -800,8 +905,8 @@ NCZ_printxcache(NCZChunkCache* cache)
     ncbytescat(buf,s);
 
     snprintf(s,sizeof(s),"\tmaxentries=%u\n\tmaxsize=%u\n\tused=%u\n\tdimsep='%c'\n",
-    	(unsigned)cache->maxentries,
-    	(unsigned)cache->maxsize,
+    	(unsigned)cache->params.nelems,
+	(unsigned)cache->params.size,
     	(unsigned)cache->used,
     	cache->dimension_separator
 	);
@@ -813,7 +918,7 @@ NCZ_printxcache(NCZChunkCache* cache)
         ncbytescat(buf,"\t\t<empty>\n");
     for(i=0;i<nclistlength(cache->mru);i++) {
 	NCZCacheEntry* e = (NCZCacheEntry*)nclistget(cache->mru,i);
-	snprintf(s,sizeof(s),"\t\t[%d] ",i);
+	snprintf(s,sizeof(s),"\t\t[%zu] ", i);
 	ncbytescat(buf,s);
 	if(e == NULL)
 	    ncbytescat(buf,"<null>");
@@ -826,5 +931,4 @@ NCZ_printxcache(NCZChunkCache* cache)
     strlcat(xs,ncbytescontents(buf),sizeof(xs));
     ncbytesfree(buf);
     fprintf(stderr,"%s\n",xs);
-//    return xs;
 }
