@@ -23,7 +23,6 @@
 #include <string.h>
 #include <math.h>
 #include <netcdf.h>
-#include <netcdf_par.h>
 #include <netcdf_meta.h>
 
 #define TEST_NAME "tst_compress"
@@ -283,7 +282,7 @@ write_meta(int ncid, int *data_varid, int s, int f, int nsd, int deflate, int u,
     {
         char data_var_name[NC_MAX_NAME + 1];
 
-        sprintf(data_var_name, "var_%d", dv);
+        snprintf(data_var_name, sizeof(data_var_name), "var_%d", dv);
         if (nc_redef(ncid)) ERR;
         if (nc_def_var(ncid, data_var_name, NC_FLOAT, NDIM4, dimid_data, &data_varid[dv])) ERR;
 
@@ -568,7 +567,7 @@ main(int argc, char **argv)
 
                         /* Use the same filename every time, so we don't
                          * create many large files, just one. ;-) */
-                        sprintf(file_name, "%s.nc", TEST_NAME);
+                        snprintf(file_name, sizeof(file_name), "%s.nc", TEST_NAME);
 
                         /* Remove the last file. Ignore errors. */
                         remove(file_name);
