@@ -102,7 +102,7 @@ main(int argc, char **argv) {
     char testfile[NC_MAX_NAME + 1];
 
     printf("\n*** Testing multidimensional variable with more than 2**32 values\n");
-    sprintf(testfile, "%s/%s", TEMP_LARGE, FILE_NAME);
+    snprintf(testfile, sizeof(testfile), "%s/%s", TEMP_LARGE, FILE_NAME);
     for (i = NC_FORMAT_CLASSIC; i <= NUM_FORMATS; i++)
     {
        printf("*** testing format %d file with short variable with > 2**32 values...", i);
