@@ -5,6 +5,8 @@
 
 #include "manyurls.h"
 
+#undef VERBOSE
+
 int main()
 {
     int i,ncid;
@@ -14,13 +16,17 @@ int main()
 	    char* tp = *p;	
             int mode = 0;
             int status = -1;
+#ifdef VERBOSE
 	    printf("Opening: %s\n",tp);
+#endif
             status = nc_open(tp, mode, &ncid);
 	    switch(status) {
 	    case NC_NOERR:
 		break;
 	    case NC_ENOTFOUND:
+#ifdef VERBOSE
 	        printf("{%d} %s\n",i,tp);
+#endif
 		status = NC_NOERR;
 		break;
 	    default:
