@@ -116,16 +116,16 @@ computewhich() { # set REMOTETESTS and constrained
 constrain() {
   T="$1;;" # add semicolons to fake out the cut command
   # see if we are using constraints will set testname and ce and testno and constrained
-  testname=`echo -n $T | cut "-d;" -f1`
-  testno=`echo -n $T | cut "-d;" -f2`
-  ce=`echo -n $T | cut "-d;" -f3`
+  testname=`echon $T | cut "-d;" -f1`
+  testno=`echon $T | cut "-d;" -f2`
+  ce=`echon $T | cut "-d;" -f3`
   if test "x$ce" = x ; then constrained=0; else constrained=1; fi
 }
 
 setcache() {
   CACHE="[cache]"
   if test "x${NOCACHETESTS}" != x ; then
-    if IGNORE=`echo -n " ${NOCACHETESTS} " | fgrep " $1 "`; then CACHE=; fi
+    if IGNORE=`echon " ${NOCACHETESTS} " | fgrep " $1 "`; then CACHE=; fi
   fi
   PARAMS="${PARAMS}${CACHE}"
 }
@@ -133,7 +133,7 @@ setcache() {
 setprefetch() {
   PREFETCH="[prefetch]"
   if test "x${NOPREFETCHTESTS}" != x ; then
-    if IGNORE=`echo -n " ${NOPREFETCHTESTS} " | fgrep " $1 "`; then PREFETCH=; fi
+    if IGNORE=`echon " ${NOPREFETCHTESTS} " | fgrep " $1 "`; then PREFETCH=; fi
   fi
   PARAMS="${PARAMS}${PREFETCH}"
 }
