@@ -11,6 +11,7 @@
 #include <nc_tests.h>
 #include "err_macros.h"
 #include <netcdf.h>
+#include <stddef.h>
 #include <stdlib.h>
 
 #define FILE_NAME_1 "tst_solar_1.nc"
@@ -188,7 +189,7 @@ main(int argc, char **argv)
 #define ATT_NAME3 "for_testing_unsigned_short_attribute_bug"
    {
       int ncid;
-      int i, j;
+      size_t i, j;
       nc_type typeid;
       nc_vlen_t data[DIM_LEN];
       int *phoney;
@@ -226,8 +227,8 @@ main(int argc, char **argv)
 #define ATT_NAME_CMP "my_favorite_wind_speeds"
 #define COMPOUND_NAME "wind_vector"
 #define NUM_FAVS 3
-#define U_VALUE 13.3
-#define V_VALUE 12.2
+#define U_VALUE 13.3f
+#define V_VALUE 12.2f
 
    {
       int ncid;
