@@ -5,8 +5,8 @@
  *********************************************************************/
 
 #include "includes.h"
-#include "nc_iter.h"
 #include <ctype.h>	/* for isprint() */
+#include <stddef.h>
 
 #ifdef ENABLE_C
 
@@ -38,22 +38,22 @@ static void genc_writeattr(Generator*,Symbol*,Bytebuffer*,int,size_t*,size_t*);
 void
 genc_netcdf(void)
 {
-    int idim, ivar, iatt, maxdims;
-    int ndims, nvars, natts, ngatts;
+    size_t idim, ivar, iatt;
+    int maxdims;
     char* cmode_string;
     const char *filename = rootgroup->file.filename;
 
 #ifdef USE_NETCDF4
-    int igrp,ityp, ngrps, ntyps;
+    size_t igrp, ityp;
 #endif
 
-    ndims = listlength(dimdefs);
-    nvars = listlength(vardefs);
-    natts = listlength(attdefs);
-    ngatts = listlength(gattdefs);
+    size_t ndims = listlength(dimdefs);
+    size_t nvars = listlength(vardefs);
+    size_t natts = listlength(attdefs);
+    size_t ngatts = listlength(gattdefs);
 #ifdef USE_NETCDF4
-    ngrps = listlength(grpdefs);
-    ntyps = listlength(typdefs);
+    size_t ngrps = listlength(grpdefs);
+    size_t ntyps = listlength(typdefs);
 #endif /*USE_NETCDF4*/
 
     /* wrap in main program */
@@ -96,11 +96,14 @@ genc_netcdf(void)
         for(ivar=0;ivar<nvars;ivar++) {
             Bytebuffer* tmp = bbNew();
             Symbol* var = (Symbol*)listget(vardefs,ivar);
-            Specialdata* special = var->var.special;
+            Specialdata* special = &var->var.special;
             if(special->flags & _CHUNKSIZES_FLAG) {
                 int i;
                 size_t* chunks = special->_ChunkSizes;
-                if(special->nchunks == 0 || chunks == NULL) continue;
+                if(special->nchunks == 0 || chunks == NULL) {
+                    bbFree(tmp);
+                    continue;
+                }
                 bbClear(tmp);
                 for(i=0;i<special->nchunks;i++) {
                     bbprintf(tmp,"%s%ld",
@@ -117,14 +120,14 @@ genc_netcdf(void)
 		for(k=0;k<special->nfilters;k++) {
 		    int i;
 		    size_t nparams;
-		    unsigned int* params;
-		    NC4_Filterspec* nfs = (NC4_Filterspec*)special->_Filters[k];
+		    unsigned int* params = NULL;
+		    NC_H5_Filterspec* nfs = special->_Filters[k];
 		    if(nfs->nparams == 0 || nfs->params == NULL) continue;
 	   	    bbClear(tmp);
 		    nparams = nfs->nparams;
 		    params = nfs->params;
 		    for(i=0;i<nparams;i++) {
-                        bbprintf(tmp,"%s%luU",
+                        bbprintf(tmp,"%s%uU",
                             (i == 0?"":", "),params[i]);
 		    }
                     bbprintf0(stmt,"static unsigned int %s_%d_filterparams[%d] = {",
@@ -142,14 +145,16 @@ genc_netcdf(void)
 
     /* Now construct the main procedures*/
     codeline("void");
-    codeline("check_err(const int stat, const int line, const char *file) {");
+    codeline("check_err(const int stat, int line, const char* file, const char* func) {");
     codelined(1,"if (stat != NC_NOERR) {");
-    codelined(2,"(void)fprintf(stderr,\"line %d of %s: %s\\n\", line, file, nc_strerror(stat));");
+    codelined(2,"(void)fprintf(stderr,\"line %d of %s.%s: %s\\n\", line, file, func, nc_strerror(stat));");
     codelined(2,"fflush(stderr);");
     codelined(2,"exit(1);");
     codelined(1,"}");
     codeline("}");
     codeline("");
+    codeline("#define CHECK_ERR(err) check_err(err, __LINE__, __FILE__, __func__)");
+    codeline("");
     codeline("int");
     bbprintf0(stmt,"%s() {/* create %s */\n", mainname, filename);
     codedump(stmt);
@@ -281,7 +286,7 @@ genc_netcdf(void)
     bbprintf0(stmt,"    stat = nc_create(\"%s\", %s, &ncid);\n",
 		 filename,cmode_string);
     codedump(stmt);
-    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+    codelined(1,"CHECK_ERR(stat);");
     codeflush();
 
 #ifdef USE_NETCDF4
@@ -305,7 +310,7 @@ genc_netcdf(void)
 		groupncid(gsym->container),
 		gsym->name, groupncid(gsym));
 	    codedump(stmt);
-	    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	    codelined(1,"CHECK_ERR(stat);");
 	}
         codeflush();
     }
@@ -341,7 +346,7 @@ genc_netcdf(void)
                           dimncid(dsym));
 	    }
 	    codedump(stmt);
-	    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	    codelined(1,"CHECK_ERR(stat);");
        }
     }
     codeflush();
@@ -375,7 +380,7 @@ genc_netcdf(void)
 			(dimset->ndims == 0?"0":poolcat(cname(vsym),"_dims")),
 			varncid(vsym));
 	    codedump(stmt);
-	    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	    codelined(1,"CHECK_ERR(stat);");
 #ifdef USE_NETCDF4
 	    genc_definespecialattributes(vsym);
 #endif /*USE_NETCDF4*/
@@ -411,14 +416,14 @@ genc_netcdf(void)
         codelined(1,"/* don't initialize variables with fill values */");
 	bbindent(stmt,1);
 	bbprintf0(stmt,"stat = nc_set_fill(%s, NC_NOFILL, 0);",groupncid(rootgroup));
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
     }
 
     codeline("");
     codelined(1,"/* leave define mode */");
     bbprintf0(stmt,"    stat = nc_enddef (%s);\n",groupncid(rootgroup));
     codedump(stmt);
-    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+    codelined(1,"CHECK_ERR(stat);");
     codeflush();
 
     if(!header_only) {
@@ -450,7 +455,7 @@ genc_defineglobalspecials(void)
 static void
 genc_definespecialattributes(Symbol* vsym)
 {
-    Specialdata* special = vsym->var.special;
+    Specialdata* special = &vsym->var.special;
     if(usingclassic) return;
     if(special->flags & _STORAGE_FLAG) {
         const char* storage = NULL;
@@ -473,7 +478,7 @@ genc_definespecialattributes(Symbol* vsym)
             codedump(stmt);
         }
         codeline(");");
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
     }
     if(special->flags & _FLETCHER32_FLAG) {
         bbprintf0(stmt,
@@ -482,7 +487,7 @@ genc_definespecialattributes(Symbol* vsym)
                 varncid(vsym),
                 special->_Fletcher32);
         codedump(stmt);
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
     }
     if(special->flags & (_DEFLATE_FLAG | _SHUFFLE_FLAG)) {
         bbprintf0(stmt,
@@ -493,7 +498,7 @@ genc_definespecialattributes(Symbol* vsym)
                 (special->_DeflateLevel >= 0?1:0),
                 (special->_DeflateLevel >= 0?special->_DeflateLevel:0));
         codedump(stmt);
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
     }
     if(special->flags & _ENDIAN_FLAG) {
         bbprintf0(stmt,
@@ -504,7 +509,7 @@ genc_definespecialattributes(Symbol* vsym)
                                                     :"NC_ENDIAN_BIG")
                 );
         codedump(stmt);
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
     }
     if(special->flags & _NOFILL_FLAG) {
         bbprintf0(stmt,
@@ -514,18 +519,18 @@ genc_definespecialattributes(Symbol* vsym)
                 (special->_Fill?"NC_FILL":"NC_NOFILL")
                 );
         codedump(stmt);
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
     }
     if(special->flags & _FILTER_FLAG) {
 	int k;
 	for(k=0;k<special->nfilters;k++) {
-	     NC4_Filterspec* nfs = (NC4_Filterspec*)special->_Filters[k];
+	     NC_H5_Filterspec* nfs = special->_Filters[k];
 	     bbprintf0(stmt,
-	                "    stat = nc_def_var_filter(%s, %s, %u, %u, ",
+	                "    stat = nc_def_var_filter(%s, %s, %u, %lu, ",
 	                groupncid(vsym->container),
 	                varncid(vsym),
 			nfs->filterid,
-			nfs->nparams
+			(unsigned long)nfs->nparams
 			);
 	    codedump(stmt);
 	    if(nfs->nparams == 0 || nfs->params == NULL)
@@ -535,9 +540,25 @@ genc_definespecialattributes(Symbol* vsym)
 	        codedump(stmt);
 	    }
 	    codeline(");");
-            codelined(1,"check_err(stat,__LINE__,__FILE__);");
+            codelined(1,"CHECK_ERR(stat);");
 	}
     }
+    if(special->flags & (_QUANTIZEBG_FLAG | _QUANTIZEGBR_FLAG)) {
+    	const char* alg = NULL;
+	switch(special->_Quantizer) {
+	case NC_QUANTIZE_BITGROOM: alg = "NC_QUANTIZE_BITGROOM";
+	case NC_QUANTIZE_GRANULARBR: alg = "NC_QUANTIZE_GRANULARBR";
+	default: alg = "NC_NOQUANTIZE";
+	}
+        bbprintf0(stmt,
+                "    stat = nc_def_var_quantize(%s, %s, %s, %d);\n",
+                groupncid(vsym->container),
+                varncid(vsym),
+		alg, special->_NSD
+                );
+        codedump(stmt);
+        codelined(1,"CHECK_ERR(stat);");
+    }
 }
 #endif /*USE_NETCDF4*/
 
@@ -546,7 +567,7 @@ genc_close(void)
 {
     bbprintf0(stmt,"%sstat = nc_close(%s);\n",indented(1),groupncid(rootgroup));
     codedump(stmt);
-    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+    codelined(1,"CHECK_ERR(stat);");
 #ifndef vms
     codelined(1,"return 0;");
 #else
@@ -744,7 +765,7 @@ ctypename(Symbol* tsym)
 static void
 definectype(Symbol* tsym)
 {
-    int i,j;
+    size_t i,j;
 
     ASSERT(tsym->objectclass == NC_TYPE);
     switch (tsym->subclass) {
@@ -824,7 +845,7 @@ Generate the C code for defining a given type
 static void
 genc_deftype(Symbol* tsym)
 {
-    int i;
+    size_t i;
 
     ASSERT(tsym->objectclass == NC_TYPE);
     switch (tsym->subclass) {
@@ -837,7 +858,7 @@ genc_deftype(Symbol* tsym)
 		tsym->name,
 		typencid(tsym));
 	codedump(stmt);
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
 	break;
     case NC_ENUM:
 	codelined(1,"{");
@@ -852,7 +873,7 @@ genc_deftype(Symbol* tsym)
 		tsym->name,
 		typencid(tsym));
 	codedump(stmt);
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
 	for(i=0;i<listlength(tsym->subnodes);i++) {
 	    Symbol* econst = (Symbol*)listget(tsym->subnodes,i);
 	    Bytebuffer* econststring = bbNew();
@@ -880,7 +901,7 @@ genc_deftype(Symbol* tsym)
 		typencid(tsym->typ.basetype),
 		typencid(tsym));
 	codedump(stmt);
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
 	break;
     case NC_COMPOUND:
 	bbprintf0(stmt,"%sstat = nc_def_compound(%s, sizeof(%s), \"%s\", &%s);",
@@ -890,7 +911,7 @@ genc_deftype(Symbol* tsym)
 		escapifyname(tsym->name),
 		typencid(tsym));
 	codedump(stmt);
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
 	/* Generate the field dimension constants*/
 	codelined(1,"{");
 	for(i=0;i<listlength(tsym->subnodes);i++) {
@@ -942,7 +963,7 @@ genc_deftype(Symbol* tsym)
 		    typencid(efield->typ.basetype));
 	    }
 	    codedump(stmt);
-	    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	    codelined(1,"CHECK_ERR(stat);");
 	}
 	codelined(1,"}");
 	break;
@@ -1012,8 +1033,7 @@ genc_writevar(Generator* generator, Symbol* vsym, Bytebuffer* code,
     /* Dump any vlen decls first */
     generator_getstate(generator,(void**)&vlendecls);
     if(vlendecls != NULL && listlength(vlendecls) > 0) {
-	int i;
-	for(i=0;i<listlength(vlendecls);i++) {
+	for(size_t i=0;i<listlength(vlendecls);i++) {
 	   Bytebuffer* decl = (Bytebuffer*)listget(vlendecls,i);
 	   codelined(1,bbContents(decl));
 	   bbFree(decl);
@@ -1040,7 +1060,7 @@ genc_writevar(Generator* generator, Symbol* vsym, Bytebuffer* code,
 		varncid(vsym),
 		cname(vsym));
         codedump(stmt);
-        codelined(1,"check_err(stat,__LINE__,__FILE__);");
+        codelined(1,"CHECK_ERR(stat);");
 	codeflush();
     } else { /* rank > 0 */
 	int i;
@@ -1102,7 +1122,7 @@ genc_writevar(Generator* generator, Symbol* vsym, Bytebuffer* code,
 			cname(vsym),
 			cname(vsym));
 	codedump(stmt);
-	codelined(1,"check_err(stat,__LINE__,__FILE__);");
+	codelined(1,"CHECK_ERR(stat);");
 
     }
     /* end defined block*/
@@ -1134,8 +1154,7 @@ genc_writeattr(Generator* generator, Symbol* asym, Bytebuffer* code,
         List* vlendecls;
         generator_getstate(generator,(void**)&vlendecls);
         if(vlendecls != NULL && listlength(vlendecls) > 0) {
-            int i;
-            for(i=0;i<listlength(vlendecls);i++) {
+            for(size_t i=0;i<listlength(vlendecls);i++) {
                 Bytebuffer* decl = (Bytebuffer*)listget(vlendecls,i);
                 codelined(1,bbContents(decl));
                 bbFree(decl);
@@ -1255,7 +1274,7 @@ genc_writeattr(Generator* generator, Symbol* asym, Bytebuffer* code,
 	break;
     }
 
-    codelined(1,"check_err(stat,__LINE__,__FILE__);");
+    codelined(1,"CHECK_ERR(stat);");
     codelined(1,"}");
 }
 
