@@ -2,10 +2,15 @@
 #include "config.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
@@ -18,22 +23,22 @@
 #include "ncpathmgr.h"
 #include "nclog.h"
 
-#ifdef HAVE_HDF5_H
+#ifdef USE_HDF5
 #include <hdf5.h>
 #include <H5DSpublic.h>
 #endif
 
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
 #include "zincludes.h"
 #endif
 
 #undef DEBUG
 
 /* Short Aliases */
-#ifdef HDF5_SUPPORTS_PAR_FILTERS
+#ifdef USE_HDF5
 #define H5
 #endif
-#ifdef ENABLE_NCZARR
+#ifdef NETCDF_ENABLE_NCZARR
 #define NZ
 #endif
 
@@ -43,7 +48,9 @@ typedef struct Format {
     char var_name[NC_MAX_NAME];
     int fillvalue;
     int debug;
-    int rank;
+    int linear;
+    int holevalue;
+    size_t rank;
     size_t dimlens[NC_MAX_VAR_DIMS];
     size_t chunklens[NC_MAX_VAR_DIMS];
     size_t chunkcounts[NC_MAX_VAR_DIMS];
@@ -64,7 +71,7 @@ typedef struct Odometer {
 #define ceildiv(x,y) (((x) % (y)) == 0 ? ((x) / (y)) : (((x) / (y)) + 1))
 
 static char* captured[4096];
-static int ncap = 0;
+static size_t ncap = 0;
 
 extern int nc__testurl(const char*,char**);
 
@@ -82,7 +89,7 @@ usage(int err)
      if(err != 0) {
 	 fprintf(stderr,"Error: (%d) %s\n",err,nc_strerror(err));
      }
-     fprintf(stderr,"usage: ncdumpchunks -v <var> <file> \n");
+     fprintf(stderr,"usage: ncdumpchunks [-b] -v <var> <file> \n");
      fflush(stderr);
      exit(1);
 }
@@ -107,7 +114,7 @@ printvector(int rank, size_t* vec)
 void
 cleanup(void)
 {
-    int i;
+    size_t i;
     for(i=0;i<ncap;i++)
         if(captured[i]) free(captured[i]);
 }
@@ -115,7 +122,7 @@ cleanup(void)
 Odometer*
 odom_new(size_t rank, const size_t* stop, const size_t* max)
 {
-     int i;
+     size_t i;
      Odometer* odom = NULL;
      if((odom = calloc(1,sizeof(Odometer))) == NULL)
 	 return NULL;
@@ -144,7 +151,7 @@ odom_more(Odometer* odom)
 int
 odom_next(Odometer* odom)
 {
-     size_t i;
+     int i;
      for(i=odom->rank-1;i>=0;i--) {
 	 odom->index[i]++;
 	 if(odom->index[i] < odom->stop[i]) break;
@@ -164,8 +171,7 @@ odom_indices(Odometer* odom)
 size_t
 odom_offset(Odometer* odom)
 {
-     size_t offset;
-     int i;
+     size_t i,offset;
 
      offset = 0;
      for(i=0;i<odom->rank;i++) {
@@ -214,7 +220,7 @@ chunk_key(int format->rank, size_t* indices)
 void
 setoffset(Odometer* odom, size_t* chunksizes, size_t* offset)
 {
-     int i;
+     size_t i;
      for(i=0;i<odom->rank;i++)
 	 offset[i] = odom->index[i] * chunksizes[i];
 }
@@ -225,12 +231,42 @@ printindent(size_t indent)
     while(indent-- > 0) printf(" ");
 }
 
+int
+printchunklinear(Format* format, int* chunkdata, size_t indent)
+{
+    size_t i;
+    for(i=0;i<format->chunkprod;i++) {
+	if(chunkdata[i] == format->fillvalue)
+	    printf(" _");
+	else
+	    printf(" %02d", chunkdata[i]);
+    }
+    printf("\n");
+    return NC_NOERR;
+}
+
+static void
+printchunk2d(Format* format, int* chunkdata, size_t indent)
+{
+    size_t pos;
+    size_t row;
+    pos = 0;
+    for(row=0;row<format->chunklens[0];row++) {
+	size_t col;
+        if(row > 0) printindent(indent);
+	for(col=0;col<format->chunklens[1];col++,pos++) {
+            if(chunkdata[pos] == format->fillvalue) printf(" _"); else printf(" %02d", chunkdata[pos]);
+	}
+        printf("\n");
+    }	
+}
+
 static void
 printchunk(Format* format, int* chunkdata, size_t indent)
 {
-    int k[3];
+    size_t k[3];
     int rank = format->rank;
-    unsigned cols[3], pos;
+    size_t cols[3], pos;
     size_t* chl = format->chunklens;
 
     memset(cols,0,sizeof(cols));
@@ -268,13 +304,19 @@ printchunk(Format* format, int* chunkdata, size_t indent)
 	    if(k[1] > 0) printf(" |");
 	    for(k[2]=0;k[2]<cols[2];k[2]++) {
 		if(format->xtype == NC_UBYTE) {
-		    int l;
+		    size_t l;
 		    unsigned char* bchunkdata = (unsigned char*)(&chunkdata[pos]);
 		    for(l=0;l<sizeof(int);l++) {
-                        printf(" %02u", bchunkdata[l]);
+			if(bchunkdata[l] == format->fillvalue)
+                            printf(" _");
+			else
+                            printf(" %02u", bchunkdata[l]);
 		    }
 		} else {
-                    printf(" %02d", chunkdata[pos]);
+		    if(chunkdata[pos] == format->fillvalue)
+			printf(" _");
+		    else
+                        printf(" %02d", chunkdata[pos]);
 		}
 		pos++;
 	    }
@@ -297,12 +339,12 @@ dump(Format* format)
 {
     void* chunkdata = NULL; /*[CHUNKPROD];*/
     Odometer* odom = NULL;
-    int r;
+    size_t r;
     size_t offset[NC_MAX_VAR_DIMS];
     int holechunk = 0;
     char sindices[64];
 #ifdef H5
-    int i;
+    size_t i;
     hid_t fileid, grpid, datasetid;
     hid_t dxpl_id = H5P_DEFAULT; /*data transfer property list */
     unsigned int filter_mask = 0;
@@ -346,7 +388,7 @@ dump(Format* format)
 
      if((chunkdata = calloc(sizeof(int),format->chunkprod))==NULL) usage(NC_ENOMEM);
 
-     printf("rank=%d dims=(%s) chunks=(%s)\n",format->rank,printvector(format->rank,format->dimlens),
+     printf("rank=%zu dims=(%s) chunks=(%s)\n",format->rank,printvector(format->rank,format->dimlens),
                                                            printvector(format->rank,format->chunklens));
 
      while(odom_more(odom)) {
@@ -387,11 +429,11 @@ dump(Format* format)
 	default: usage(NC_EINVAL);
 	}
 	if(holechunk) {
-	    /* Hole chunk: use fillvalue */
+	    /* Hole chunk: use holevalue */
 	    size_t i = 0;
 	    int* idata = (int*)chunkdata;
 	    for(i=0;i<format->chunkprod;i++)
-	        idata[i] = format->fillvalue;
+	        idata[i] = format->holevalue;
 	}
 	sindices[0] = '\0';
 	for(r=0;r<format->rank;r++) {
@@ -401,7 +443,12 @@ dump(Format* format)
 	}
 	strcat(sindices," =");
 	printf("%s",sindices);
-	printchunk(format,chunkdata,strlen(sindices));
+	if(format->linear)
+             printchunklinear(format,chunkdata,strlen(sindices));
+        else if(format->rank == 2)
+  	    printchunk2d(format,chunkdata,strlen(sindices));
+	else
+   	    printchunk(format,chunkdata,strlen(sindices));
 	fflush(stdout);
 	odom_next(odom);
      }
@@ -459,29 +506,38 @@ done:
 int
 main(int argc, char** argv)
 {
-    int i,stat = NC_NOERR;
+    int stat = NC_NOERR;
+    size_t i;
     Format format;
     int ncid, varid, dimids[NC_MAX_VAR_DIMS];
     int vtype, storage;
     int mode;
     int c;
+    int r;
 
     memset(&format,0,sizeof(format));
 
     /* Init some format fields */
     format.xtype = NC_INT;
-
-    while ((c = getopt(argc, argv, "bv:DT:")) != EOF) {
+    format.holevalue = -2;
+    
+    while ((c = getopt(argc, argv, "bhv:DT:L")) != EOF) {
     switch(c) {
 	case 'b':
 	    format.xtype = NC_UBYTE;
-	    break;
+	    break;	
+	case 'h':
+	    usage(0);
+	    break;	
 	case 'v':
 	    strcpy(format.var_name,optarg);
-	    break;
+	    break;	
 	case 'D':
 	    format.debug = 1;
 	    break;
+	case 'L':
+	    format.linear = 1;
+	    break;
 	case 'T':
 	    nctracelevel(atoi(optarg));
 	    break;
@@ -523,7 +579,8 @@ main(int argc, char** argv)
 
     /* Get the info about the var */
     if((stat=nc_inq_varid(ncid,format.var_name,&varid))) usage(stat);
-    if((stat=nc_inq_var(ncid,varid,NULL,&vtype,&format.rank,dimids,NULL))) usage(stat);
+    if((stat=nc_inq_var(ncid,varid,NULL,&vtype,&r,dimids,NULL))) usage(stat);
+    format.rank = (size_t)r;
     if(format.rank == 0) usage(NC_EDIMSIZE);
     if((stat=nc_inq_var_chunking(ncid,varid,&storage,format.chunklens))) usage(stat);
     if(storage != NC_CHUNKED) usage(NC_EBADCHUNK);
