@@ -6,18 +6,18 @@
 #ifndef UT_UTIL_H
 #define UT_UTIL_H
 
+#include <stddef.h>
 extern int parseslices(const char* s0, int* nslicesp, NCZSlice* slices);
 extern int parsedimdef(const char* s0, Dimdef** defp);
 extern int parsevardef(const char* s0, NClist* dimdefs, Vardef** varp);
 extern int parsestringvector(const char* s0, int stopchar, char*** namesp);
-extern int parseintvector(const char* s0, int typelen, void** vectorp);
 extern void freedimdefs(NClist* defs);
 extern void freevardefs(NClist* defs);
 extern void freeranges(NCZChunkRange* ranges);
 extern void freeslices(NCZSlice* slices);
 extern void freestringvec(char** vec);
 extern void freeprojvector(int rank, NCZProjection** vec);
-extern int ut_typesize(nc_type t);
+extern size_t ut_typesize(nc_type t);
 extern nc_type ut_typeforname(const char* tname);
 extern NCZM_IMPL kind2impl(const char* kind);
 extern const char* impl2kind(NCZM_IMPL impl);
