@@ -10,8 +10,6 @@
 #include <sys/stat.h>
 #endif
 
-extern int fileno(FILE*);
-
 #include "ocinternal.h"
 #include "ocdebug.h"
 
@@ -40,7 +38,7 @@ static void
 tabto(int pos, NCbytes* buffer)
 {
     int bol,len,pad;
-    len = ncbyteslength(buffer);
+    len = (int)ncbyteslength(buffer);
     /* find preceding newline */
     for(bol=len-1;;bol--) {
 	int c = ncbytesget(buffer,(size_t)bol);
@@ -73,7 +71,7 @@ dumpocnode1(OCnode* node, int depth)
 	if(node->name == NULL) OCPANIC("prim without name");
 	fprintf(stdout,"%s %s",octypetostring(node->etype),node->name);
 	dumpdimensions(node);
-	fprintf(stdout," &%lx",(unsigned long)node);
+	fprintf(stdout," &%p",node);
 	fprintf(stdout,"\n");
     } break;
 
@@ -91,7 +89,7 @@ dumpocnode1(OCnode* node, int depth)
 	fprintf(stdout,"struct %s",
 		(node->name?node->name:""));
 	dumpdimensions(node);
-	fprintf(stdout," &%lx",(unsigned long)node);
+	fprintf(stdout," &%p",node);
 	fprintf(stdout,"\n");
 	for(n=0;n<nclistlength(node->subnodes);n++) {
 	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
@@ -103,7 +101,7 @@ dumpocnode1(OCnode* node, int depth)
 	fprintf(stdout,"sequence %s",
 		(node->name?node->name:""));
 	dumpdimensions(node);
-	fprintf(stdout," &%lx",(unsigned long)node);
+	fprintf(stdout," &%p",node);
 	fprintf(stdout,"\n");
 	for(n=0;n<nclistlength(node->subnodes);n++) {
 	    dumpocnode1((OCnode*)nclistget(node->subnodes,n),depth+1);
@@ -116,7 +114,7 @@ dumpocnode1(OCnode* node, int depth)
 	fprintf(stdout,"grid %s",
 		(node->name?node->name:""));
 	dumpdimensions(node);
-	fprintf(stdout," &%lx",(unsigned long)node);
+	fprintf(stdout," &%p",node);
 	fprintf(stdout,"\n");
 	fprintf(stdout,"%sarray:\n",dent2(depth+1));
 	dumpocnode1((OCnode*)nclistget(node->subnodes,0),depth+2);
@@ -135,7 +133,7 @@ dumpocnode1(OCnode* node, int depth)
 	    if(n > 0) fprintf(stdout,",");
 	    fprintf(stdout," %s",value);
 	}
-	fprintf(stdout," &%lx",(unsigned long)node);
+	fprintf(stdout," &%p",node);
 	fprintf(stdout,"\n");
     } break;
 
@@ -235,7 +233,7 @@ addfield(char* field, size_t llen, char* line, int align)
     int len,rem;
     strlcat(line,"|",llen);
     strlcat(line,field,llen);
-    len = strlen(field);
+    len = (int)strlen(field);
     rem = (align - len);
     while(rem-- > 0) strlcat(line," ",llen);
 }
@@ -261,27 +259,27 @@ dumpfield(size_t index, char* n8, int isxdr)
     line[0] = '\0';
 
     /* offset */
-    sprintf(tmp,"%6zd",index);
+    snprintf(tmp,sizeof(tmp),"%6zd",index);
     addfield(tmp,sizeof(line),line,5);
 
     memcpy(form.cv,n8,4);
 
     /* straight hex*/
-    sprintf(tmp,"%08x",form.uv);
+    snprintf(tmp,sizeof(tmp),"%08x",form.uv);
     addfield(tmp,sizeof(line),line,8);
 
     if(isxdr) {swapinline32(&form.uv);}
 
     /* unsigned integer */
-    sprintf(tmp,"%12u",form.uv);
+    snprintf(tmp,sizeof(tmp),"%12u",form.uv);
     addfield(tmp,sizeof(line),line,12);
 
     /* signed integer */
-    sprintf(tmp,"%12d",form.sv);
+    snprintf(tmp,sizeof(tmp),"%12d",form.sv);
     addfield(tmp,sizeof(line),line,12);
 
     /* float */
-    sprintf(tmp,"%#g",form.fv);
+    snprintf(tmp,sizeof(tmp),"%#g",form.fv);
     addfield(tmp,sizeof(line),line,12);
 
     /* char[4] */
@@ -305,7 +303,7 @@ dumpfield(size_t index, char* n8, int isxdr)
     /* double */
     memcpy(dform.cv,n8,(size_t)(2*XDRUNIT));
     if(isxdr) xxdrntohdouble(dform.cv,&dform.d);
-    sprintf(tmp,"%#g",dform.d);
+    snprintf(tmp,sizeof(tmp),"%#g",dform.d);
     addfield(tmp,sizeof(line),line,12);
 
     fprintf(stdout,"%s\n",line);
@@ -314,7 +312,7 @@ dumpfield(size_t index, char* n8, int isxdr)
 static void
 typedmemorydump(char* memory, size_t len, int fromxdr)
 {
-    unsigned int i,count,rem;
+    unsigned int i,rem;
     char line[1024];
     char* pmem;
     char mem[8];
@@ -333,7 +331,7 @@ typedmemorydump(char* memory, size_t len, int fromxdr)
     strlcat(line,"\n",sizeof(line));
     fprintf(stdout,"%s",line);
 
-    count = (len / sizeof(int));
+    size_t count = (len / sizeof(int));
     rem = (len % sizeof(int));
 
     for(pmem=memory,i=0;i<count;i++,pmem+=4) {
@@ -355,7 +353,7 @@ typedmemorydump(char* memory, size_t len, int fromxdr)
 static void
 simplememorydump(char* memory, size_t len, int fromxdr)
 {
-    unsigned int i,count,rem;
+    unsigned int i,rem;
     int* imemory;
     char tmp[32];
     char line[1024];
@@ -369,7 +367,7 @@ simplememorydump(char* memory, size_t len, int fromxdr)
     addfield("!XDR (hex)",sizeof(line),line,10);
     fprintf(stdout,"%s\n",line);
 
-    count = (len / sizeof(int));
+    size_t count = (len / sizeof(int));
     rem = (len % sizeof(int));
     if(rem != 0)
 	fprintf(stderr,"ocdump: |mem|%%4 != 0\n");
@@ -380,11 +378,11 @@ simplememorydump(char* memory, size_t len, int fromxdr)
 	unsigned int v = vx;
 	if(!xxdr_network_order) swapinline32(&v);
         line[0] = '\0';
-        sprintf(tmp,"%6d",i);
+        snprintf(tmp,sizeof(tmp),"%6d",i);
         addfield(tmp,sizeof(line),line,6);
-        sprintf(tmp,"%08x",vx);
+        snprintf(tmp,sizeof(tmp),"%08x",vx);
         addfield(tmp,sizeof(line),line,9);
-        sprintf(tmp,"%08x",v);
+        snprintf(tmp,sizeof(tmp),"%08x",v);
         addfield(tmp,sizeof(line),line,10);
         fprintf(stdout,"%s\n",line);
     }
@@ -411,7 +409,6 @@ ocreadfile(FILE* file, off_t datastart, char** memp, size_t* lenp)
 {
     char* mem = NULL;
     size_t len;
-    long red;
     struct stat stats;
     long pos;
     OCerror stat = OC_NOERR;
@@ -435,14 +432,14 @@ ocreadfile(FILE* file, off_t datastart, char** memp, size_t* lenp)
 	stat = OC_ERCFILE;
 	goto done;
     }
-    len = stats.st_size;
-    len -= datastart;
+    len = (size_t)stats.st_size;
+    len -= (size_t)datastart;
 
     mem = (char*)calloc(len+1,1);
     if(mem == NULL) {stat = OC_ENOMEM; goto done;}
 
     /* Read only the data part */
-    red = fread(mem,1,len,file);
+    size_t red = fread(mem,1,len,file);
     if(red < len) {
 	fprintf(stderr,"ocreadfile: short file\n");
 	stat = OC_ERCFILE;
@@ -483,8 +480,8 @@ ocdd(OCstate* state, OCnode* root, int xdrencoded, int level)
     } else {
         mem = root->tree->data.memory;
         mem += root->tree->data.bod;
-        len = root->tree->data.datasize;
-        len -= root->tree->data.bod;
+        len = (size_t)root->tree->data.datasize;
+        len -= (size_t)root->tree->data.bod;
         ocdumpmemory(mem,len,xdrencoded,level);
     }
 }
@@ -496,7 +493,7 @@ ocdumpdata(OCstate* state, OCdata* data, NCbytes* buffer, int frominstance)
     OCnode* pattern = data->pattern;
     char* smode = NULL;
 
-    snprintf(tmp,sizeof(tmp),"%lx:",(unsigned long)data);
+    snprintf(tmp,sizeof(tmp),"%p:",data);
     ncbytescat(buffer,tmp);
     if(!frominstance) {
         ncbytescat(buffer," node=");
@@ -516,7 +513,7 @@ ocdumpdata(OCstate* state, OCdata* data, NCbytes* buffer, int frominstance)
         ncbytescat(buffer,tmp);
     }
     ncbytescat(buffer," container=");
-    snprintf(tmp,sizeof(tmp),"%lx",(unsigned long)data->container);
+    snprintf(tmp,sizeof(tmp),"%p",data->container);
     ncbytescat(buffer,tmp);
     ncbytescat(buffer," mode=");
     ncbytescat(buffer,(smode=ocdtmodestring(data->datamode,0)));
@@ -653,8 +650,8 @@ ocdumpdatapath(OCstate* state, OCdata* data, NCbytes* buffer)
 	}
 	if(pattern->octype == OC_Atomic) {
 	    if(pattern->array.rank > 0) {
-	        off_t xproduct = octotaldimsize(pattern->array.rank,pattern->array.sizes);
-	        snprintf(tmp,sizeof(tmp),"[0..%lu]",(unsigned long)xproduct-1);
+	        size_t xproduct = octotaldimsize(pattern->array.rank,pattern->array.sizes);
+	        snprintf(tmp,sizeof(tmp),"[0..%lu]",xproduct-1);
 	        ncbytescat(buffer,tmp);
 	    }
 	}
@@ -677,6 +674,6 @@ ocdumpdatapath(OCstate* state, OCdata* data, NCbytes* buffer)
 	ncbytescat(buffer,":");
 	ncbytescat(buffer,octypetoddsstring(pattern->etype));
     }
-    snprintf(tmp,sizeof(tmp),"->0x%0lx",(unsigned long)pathdata);
+    snprintf(tmp,sizeof(tmp),"->0x%p",pathdata);
     ncbytescat(buffer,tmp);
 }
